@@ -73,7 +73,7 @@ class Authorizer(object):
 
         # check it's active
         if domain_object.__class__ != type and hasattr(domain_object, 'state'):
-            if domain_object.state != model.State.ACTIVE:
+            if domain_object.state == model.State.DELETED:
                 return False
 
         # check if any of the roles allows the action requested
@@ -184,7 +184,6 @@ class Authorizer(object):
             user = model.User.by_name(username, autoflush=False)
         else:
             user = None
-        entity.roles.property.mapper.class_ 
         visitor = model.User.by_name(model.PSEUDO_USER__VISITOR, autoflush=False)
         logged_in = model.User.by_name(model.PSEUDO_USER__LOGGED_IN,
                                        autoflush=False)
@@ -193,8 +192,15 @@ class Authorizer(object):
             # need to use this in the queries below as if we use
             # model.UserObjectRole a cross join happens always
             # returning all the roles.  
-            role_cls = entity.roles.property.mapper.class_
-            q = q.outerjoin('roles')
+            if hasattr(entity, 'continuity'):
+                q = q.filter_by(current=True)
+                q = q.outerjoin('continuity', 'roles')
+                continuity = entity.continuity.property.mapper.class_
+                role_cls = continuity.roles.property.mapper.class_ 
+            else:
+                role_cls = entity.roles.property.mapper.class_ 
+                q = q.outerjoin('roles')
+
             if hasattr(entity, 'state'):
                 state = entity.state
             else:
@@ -209,19 +215,49 @@ class Authorizer(object):
                 q = q.filter(sa.or_(
                     sa.and_(role_cls.role==model.RoleAction.role,
                             model.RoleAction.action==action,
-                            state and state==model.State.ACTIVE),
+                            state and state!=model.State.DELETED),
                     role_cls.role==model.Role.ADMIN))
             else:
                 q = q.filter(
                     sa.and_(role_cls.role==model.RoleAction.role,
                             model.RoleAction.action==action,
-                            state and state==model.State.ACTIVE),
+                            state and state!=model.State.DELETED),
                     )
             q = q.filter(sa.or_(*filters))   
             q = q.distinct()
 
         return q
 
+    @classmethod
+    def authorized_package_relationships(cls, username,
+                                         package1,
+                                         package2=None,
+                                         relationship_type=None,
+                                         action=model.Action.READ):
+        '''For a given package(s) returns a list of relationships that
+        the specified user is allowed to do the specified action on.'''
+        # Maybe there is an sqlalchemy query to do this all in one, but
+        # it would be rather complex.
+        rels = package1.get_relationships(with_package=package2,
+                                          type=relationship_type)
+        authorized_rels = []
+        for rel in rels:
+            if cls.authorized_package_relationship(
+                username, rel.subject, rel.object, action):
+                authorized_rels.append(rel)
+        return authorized_rels
+
+    @classmethod
+    def authorized_package_relationship(cls, username,
+                                        package1,
+                                        package2,
+                                        action=model.Action.READ):
+        '''Returns a boolean - whether a user is authorized to perform the
+        specified action on a package relationship between the specified
+        packages.'''
+        return cls.is_authorized(username, action, package1) and \
+               cls.is_authorized(username, action, package2)
+
     @classmethod
     def _get_roles_query(cls, domain_obj):
         q = model.Session.query(model.UserObjectRole)
