@@ -26,16 +26,17 @@ def make_map():
     for plugin in routing_plugins:
         map = plugin.before_map(map)
         
+    map.connect('*url', controller='home', action='cors_options',
+        conditions=dict(method=['OPTIONS']))
     map.connect('home', '/', controller='home', action='index')
     map.connect('guide', config.get('guide_url', 'http://wiki.okfn.org/ckan/doc/'), _static=True)
-    map.connect('license', '/license', controller='home', action='license')
-    map.connect('/language.js', controller='home', action='language')
     map.connect('/locale', controller='home', action='locale')
     map.connect('about', '/about', controller='home', action='about')
 
     # CKAN API versioned.
     register_list = [
             'package',
+            'dataset',
             'resource',
             'tag',
             'group',
@@ -46,9 +47,9 @@ def make_map():
             ]
     register_list_str = '|'.join(register_list)
 
-    map.connect('/api/{ver:1|2}', controller='api', action='get_api')
+    map.connect('/api/{ver:1|2|3}', controller='api', action='get_api')
 
-    map.connect('/api/{ver:1|2}/search/{register}', controller='api', action='search')
+    map.connect('/api/{ver:1|2|3}/search/{register}', controller='api', action='search')
     map.connect('/api/{ver:1|2}/tag_counts', controller='api', action='tag_counts')
 
     map.connect('/api/{ver:1|2}/rest', controller='api', action='index')
@@ -91,6 +92,8 @@ def make_map():
         controller='api', action='delete',
         requirements=dict(register=register_list_str),
         conditions=dict(method=['DELETE']))
+    map.connect('/api/{ver:3}/action/{logic_function}', controller='api', action='action',
+                conditions=dict(method=['GET', 'POST']))
     map.connect('/api/{ver:1|2}/qos/throughput/',
         controller='api', action='throughput',
         requirements=dict(register=register_list_str),
@@ -104,8 +107,8 @@ def make_map():
     
     map.connect('/api/rest', controller='api', action='index')
 
-    map.connect('/api/action/{logic_function}', controller='api', action='action')
-
+    map.connect('/api/action/{logic_function}', controller='api', action='action',
+                conditions=dict(method=['GET', 'POST']))
     map.connect('/api/rest/{register}', controller='api', action='list',
         requirements=dict(register=register_list_str),
         conditions=dict(method=['GET'])
@@ -152,27 +155,33 @@ def make_map():
 
     map.connect('/api/2/util/user/autocomplete', controller='api',
         action='user_autocomplete')
-    map.connect('/api/2/util/package/create_slug', controller='api', action='create_slug',
+    map.connect('/api/2/util/is_slug_valid', controller='api', action='is_slug_valid',
                 conditions=dict(method=['GET']))
     map.connect('/api/2/util/tag/autocomplete', controller='api', action='tag_autocomplete',
                 conditions=dict(method=['GET']))
+    map.connect('/api/2/util/resource/format_autocomplete', controller='api', action='format_autocomplete',
+                conditions=dict(method=['GET']))
 
     map.connect('/api/2/util/authorizationgroup/autocomplete', controller='api',
         action='authorizationgroup_autocomplete')
 
+    map.connect('/api/util/markdown', controller='api', action='markdown')
+
     ###########
     ## /END API
     ###########
 
-    map.redirect("/packages", "/package")
-    map.redirect("/packages/{url:.*}", "/package/{url}")
-    map.connect('/package', controller='package', action='search')
+    map.redirect("/packages", "/dataset")
+    map.redirect("/packages/{url:.*}", "/dataset/{url}")
+    map.redirect("/package", "/dataset")
+    map.redirect("/package/{url:.*}", "/dataset/{url}")
+    map.connect('/dataset', controller='package', action='search')
 
     ##to get back formalchemy uncomment these lines
     ##map.connect('/package/new', controller='package_formalchemy', action='new')
     ##map.connect('/package/edit/{id}', controller='package_formalchemy', action='edit')
 
-    map.connect('/package/{action}', controller='package',
+    map.connect('/dataset/{action}', controller='package',
         requirements=dict(action='|'.join([
             'list',
             'new',
@@ -180,9 +189,9 @@ def make_map():
             'search'
             ]))
         )
-    map.connect('/package', controller='package', action='index')
-    map.connect('/package/{action}/{id}/{revision}', controller='package', action='read_ajax')
-    map.connect('/package/{action}/{id}', controller='package',
+    map.connect('/dataset', controller='package', action='index')
+    map.connect('/dataset/{action}/{id}/{revision}', controller='package', action='read_ajax')
+    map.connect('/dataset/{action}/{id}', controller='package',
         requirements=dict(action='|'.join([
         'edit',
         'authz',
@@ -191,7 +200,7 @@ def make_map():
         'history_ajax',
         ]))
         )
-    map.connect('/package/{id}', controller='package', action='read')
+    map.connect('/dataset/{id}', controller='package', action='read')
     # group
     map.redirect("/groups", "/group")
     map.redirect("/groups/{url:.*}", "/group/{url}")
@@ -247,6 +256,9 @@ def make_map():
     map.connect('/revision/diff/{id}', controller='revision', action='diff')
     map.connect('/revision/list', controller='revision', action='list')
     map.connect('/revision/{id}', controller='revision', action='read')
+
+    map.connect('ckanadmin_index', '/ckan-admin', controller='admin', action='index')
+    map.connect('ckanadmin', '/ckan-admin/{action}', controller='admin')
     
     for plugin in routing_plugins:
         map = plugin.after_map(map)
