@@ -9,11 +9,9 @@ import ckan.authz as authz
 from ckan.authz import Authorizer
 from ckan.lib.helpers import Page
 from ckan.plugins import PluginImplementations, IGroupController
-import ckan.logic.action.create as create
-import ckan.logic.action.update as update
-import ckan.logic.action.get as get
 from ckan.lib.navl.dictization_functions import DataError, unflatten, validate
 from ckan.logic import NotFound, NotAuthorized, ValidationError
+from ckan.logic import check_access, get_action
 from ckan.logic.schema import group_form_schema
 from ckan.logic import tuplize_dict, clean_dict, parse_params
 import ckan.forms
@@ -34,24 +32,32 @@ class GroupController(BaseController):
         c.is_sysadmin = Authorizer().is_sysadmin(c.user)
 
         ## This is messy as auths take domain object not data_dict
-        group = context.get('group') or c.pkg
+        context_group = context.get('group',None)
+        group = context_group or c.group
         if group:
-            c.auth_for_change_state = Authorizer().am_authorized(
-                c, model.Action.CHANGE_STATE, group)
+            try:
+                if not context_group:
+                    context['group'] = group
+                check_access('group_change_state',context)
+                c.auth_for_change_state = True
+            except NotAuthorized:
+                c.auth_for_change_state = False
 
     ## end hooks
     
     def index(self):
 
-        if not self.authorizer.am_authorized(c, model.Action.SITE_READ, model.System):
-            abort(401, _('Not authorized to see this page'))
-
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author}
 
         data_dict = {'all_fields': True}
-               
-        results = get.group_list(context,data_dict)
+
+        try:
+            check_access('site_read',context)
+        except NotAuthorized:
+            abort(401, _('Not authorized to see this page'))
+        
+        results = get_action('group_list')(context,data_dict)
 
         c.page = Page(
             collection=results,
@@ -67,12 +73,18 @@ class GroupController(BaseController):
                    'schema': self._form_to_db_schema()}
         data_dict = {'id': id}
         try:
-            c.group_dict = get.group_show(context, data_dict)
+            c.group_dict = get_action('group_show')(context, data_dict)
             c.group = context['group']
         except NotFound:
             abort(404, _('Group not found'))
         except NotAuthorized:
             abort(401, _('Unauthorized to read group %s') % id)
+        try:
+            description_formatted = ckan.misc.MarkdownFormat().to_html(group.get('description',''))
+            c.description_formatted = genshi.HTML(description_formatted)
+        except Exception, e:
+            error_msg = "<span class='inline-warning'>%s</span>" % _("Cannot render description")
+            c.description_formatted = genshi.HTML(error_msg)
         
         try:
  
@@ -97,9 +109,9 @@ class GroupController(BaseController):
                    'user': c.user or c.author, 'extras_as_string': True,
                    'schema': self._form_to_db_schema(),
                    'save': 'save' in request.params}
-
-        auth_for_create = Authorizer().am_authorized(c, model.Action.GROUP_CREATE, model.System())
-        if not auth_for_create:
+        try:
+            check_access('group_create',context)
+        except NotAuthorized:
             abort(401, _('Unauthorized to create a group'))
 
         if context['save'] and not data:
@@ -126,22 +138,26 @@ class GroupController(BaseController):
             return self._save_edit(id, context)
 
         try:
-            old_data = get.group_show(context, data_dict)
+            old_data = get_action('group_show')(context, data_dict)
             c.grouptitle = old_data.get('title')
             c.groupname = old_data.get('name')
             schema = self._db_to_form_schema()
-            if schema:
-                old_data, errors = validate(old_data, schema)
+            if schema and not data:
+                old_data, errors = validate(old_data, schema, context=context)
 
             data = data or old_data
-
+        except NotFound:
+            abort(404, _('Group not found'))
         except NotAuthorized:
             abort(401, _('Unauthorized to read group %s') % '')
 
         group = context.get("group")
+        c.group = group
 
-        am_authz = self.authorizer.am_authorized(c, model.Action.EDIT, group)
-        if not am_authz:
+
+        try:
+            check_access('group_update',context)
+        except NotAuthorized, e:
             abort(401, _('User %r not authorized to edit %s') % (c.user, id))
 
         errors = errors or {}
@@ -156,7 +172,7 @@ class GroupController(BaseController):
             data_dict = clean_dict(unflatten(
                 tuplize_dict(parse_params(request.params))))
             context['message'] = data_dict.get('log_message', '')
-            group = create.group_create(context, data_dict)
+            group = get_action('group_create')(context, data_dict)
             h.redirect_to(controller='group', action='read', id=group['name'])
         except NotAuthorized:
             abort(401, _('Unauthorized to read group %s') % '')
@@ -175,7 +191,7 @@ class GroupController(BaseController):
                 tuplize_dict(parse_params(request.params))))
             context['message'] = data_dict.get('log_message', '')
             data_dict['id'] = id
-            group = update.group_update(context, data_dict)
+            group = get_action('group_update')(context, data_dict)
             h.redirect_to(controller='group', action='read', id=group['name'])
         except NotAuthorized:
             abort(401, _('Unauthorized to read group %s') % id)
@@ -195,209 +211,30 @@ class GroupController(BaseController):
         c.groupname = group.name
         c.grouptitle = group.display_name
 
-        c.authz_editable = self.authorizer.am_authorized(c, model.Action.EDIT_PERMISSIONS, group)
+        try:
+            context = {'model':model,'user':c.user or c.author, 'group':group}
+            check_access('group_edit_permissions',context)
+            c.authz_editable = True
+            c.group = context['group']
+        except NotAuthorized:
+            c.authz_editable = False
         if not c.authz_editable:
             abort(401, gettext('User %r not authorized to edit %s authorizations') % (c.user, id))
- 
 
-        #see package.py for comments
-        def get_userobjectroles():
-            group = model.Group.get(id)
-            uors = model.Session.query(model.GroupRole).join('group').filter_by(name=group.name).all()
-            return uors
-
-        def action_save_form(users_or_authz_groups):
-            # The permissions grid has been saved
-            # which is a grid of checkboxes named user$role
-            rpi = request.params.items()
-
-            # The grid passes us a list of the users/roles that were displayed
-            submitted = [ a for (a,b) in rpi if (b == u'submitted')]
-            # and also those which were checked
-            checked = [ a for (a,b) in rpi if (b == u'on')]
-
-            # from which we can deduce true/false for each user/role combination
-            # that was displayed in the form
-            table_dict={}
-            for a in submitted:
-                table_dict[a]=False
-            for a in checked:
-                table_dict[a]=True
-
-            # now we'll split up the user$role strings to make a dictionary from 
-            # (user,role) to True/False, which tells us what we need to do.
-            new_user_role_dict={}
-            for (ur,val) in table_dict.items():
-                u,r = ur.split('$')
-                new_user_role_dict[(u,r)] = val
-               
-            # we get the current user/role assignments 
-            # and make a dictionary of them
-            current_uors = get_userobjectroles()
-
-            if users_or_authz_groups=='users':
-                current_users_roles = [( uor.user.name, uor.role) for uor in current_uors if uor.user]
-            elif users_or_authz_groups=='authz_groups':
-                current_users_roles = [( uor.authorized_group.name, uor.role) for uor in current_uors if uor.authorized_group]        
-            else:
-                assert False, "shouldn't be here"
-
-            current_user_role_dict={}
-            for (u,r) in current_users_roles:
-                current_user_role_dict[(u,r)]=True
-
-            # and now we can loop through our dictionary of desired states
-            # checking whether a change needs to be made, and if so making it
-
-            # Here we check whether someone is already assigned a role, in order
-            # to avoid assigning it twice, or attempting to delete it when it
-            # doesn't exist. Otherwise problems can occur.
-            if users_or_authz_groups=='users':
-                for ((u,r), val) in new_user_role_dict.items():
-                    if val:
-                        if not ((u,r) in current_user_role_dict):
-                            model.add_user_to_role(model.User.by_name(u),r,group)
-                    else:
-                        if ((u,r) in current_user_role_dict):
-                            model.remove_user_from_role(model.User.by_name(u),r,group)
-            elif users_or_authz_groups=='authz_groups':
-                for ((u,r), val) in new_user_role_dict.items():
-                    if val:
-                        if not ((u,r) in current_user_role_dict):
-                            model.add_authorization_group_to_role(model.AuthorizationGroup.by_name(u),r,group)
-                    else:
-                        if ((u,r) in current_user_role_dict):
-                            model.remove_authorization_group_from_role(model.AuthorizationGroup.by_name(u),r,group)
-            else:
-                assert False, "shouldn't be here"
-
-
-            # finally commit the change to the database
-            model.repo.commit_and_remove()
-            h.flash_success("Changes Saved")
-
-
-
-        def action_add_form(users_or_authz_groups):
-            # The user is attempting to set new roles for a named user
-            new_user = request.params.get('new_user_name')
-            # this is the list of roles whose boxes were ticked
-            checked_roles = [ a for (a,b) in request.params.items() if (b == u'on')]
-            # this is the list of all the roles that were in the submitted form
-            submitted_roles = [ a for (a,b) in request.params.items() if (b == u'submitted')]
-
-            # from this we can make a dictionary of the desired states
-            # i.e. true for the ticked boxes, false for the unticked
-            desired_roles = {}
-            for r in submitted_roles:
-                desired_roles[r]=False
-            for r in checked_roles:
-                desired_roles[r]=True
-
-            # again, in order to avoid either creating a role twice or deleting one which is
-            # non-existent, we need to get the users' current roles (if any)
-  
-            current_uors = get_userobjectroles()
-
-            if users_or_authz_groups=='users':
-                current_roles = [uor.role for uor in current_uors if ( uor.user and uor.user.name == new_user )]
-                user_object = model.User.by_name(new_user)
-                if user_object==None:
-                    # The submitted user does not exist. Bail with flash message
-                    h.flash_error('unknown user:' + str (new_user))
-                else:
-                    # Whenever our desired state is different from our current state, change it.
-                    for (r,val) in desired_roles.items():
-                        if val:
-                            if (r not in current_roles):
-                                model.add_user_to_role(user_object, r, group)
-                        else:
-                            if (r in current_roles):
-                                model.remove_user_from_role(user_object, r, group)
-                    h.flash_success("User Added")
-
-            elif users_or_authz_groups=='authz_groups':
-                current_roles = [uor.role for uor in current_uors if ( uor.authorized_group and uor.authorized_group.name == new_user )]
-                user_object = model.AuthorizationGroup.by_name(new_user)
-                if user_object==None:
-                    # The submitted user does not exist. Bail with flash message
-                    h.flash_error('unknown authorization group:' + str (new_user))
-                else:
-                    # Whenever our desired state is different from our current state, change it.
-                    for (r,val) in desired_roles.items():
-                        if val:
-                            if (r not in current_roles):
-                                model.add_authorization_group_to_role(user_object, r, group)
-                        else:
-                            if (r in current_roles):
-                                model.remove_authorization_group_from_role(user_object, r, group)
-                    h.flash_success("Authorization Group Added")
-
-            else:
-                assert False, "shouldn't be here"
-
-            # and finally commit all these changes to the database
-            model.repo.commit_and_remove()
-
-
-        # In the event of a post request, work out which of the four possible actions
-        # is to be done, and do it before displaying the page
-        if 'add' in request.POST:
-            action_add_form('users')
-
-        if 'authz_add' in request.POST:
-            action_add_form('authz_groups')
-
-        if 'save' in request.POST:
-            action_save_form('users')
-
-        if 'authz_save' in request.POST:
-            action_save_form('authz_groups')
-
-        # =================
-        # Display the page
-
-        # Find out all the possible roles. At the moment, any role can be
-        # associated with any object, so that's easy:
-        possible_roles = model.Role.get_all()
-
-        # get the list of users who have roles on this object, with their roles
-        uors = get_userobjectroles()
-
-        # uniquify and sort
-        users = sorted(list(set([uor.user.name for uor in uors if uor.user])))
-        authz_groups = sorted(list(set([uor.authorized_group.name for uor in uors if uor.authorized_group])))
-
-        # make a dictionary from (user, role) to True, False
-        users_roles = [( uor.user.name, uor.role) for uor in uors if uor.user]
-        user_role_dict={}
-        for u in users:
-            for r in possible_roles:
-                if (u,r) in users_roles:
-                    user_role_dict[(u,r)]=True
-                else:
-                    user_role_dict[(u,r)]=False
-
-        # and similarly make a dictionary from (authz_group, role) to True, False
-        authz_groups_roles = [( uor.authorized_group.name, uor.role) for uor in uors if uor.authorized_group]
-        authz_groups_role_dict={}
-        for u in authz_groups:
-            for r in possible_roles:
-                if (u,r) in authz_groups_roles:
-                    authz_groups_role_dict[(u,r)]=True
-                else:
-                    authz_groups_role_dict[(u,r)]=False
-
-        # pass these variables to the template for rendering
-        c.roles = possible_roles
-
-        c.users = users
-        c.user_role_dict = user_role_dict
-
-        c.authz_groups = authz_groups
-        c.authz_groups_role_dict = authz_groups_role_dict
+        current_uors = self._get_userobjectroles(id)
+        self._handle_update_of_authz(current_uors, group)
 
+        # get the roles again as may have changed
+        user_object_roles = self._get_userobjectroles(id)
+        self._prepare_authz_info_for_render(user_object_roles)
         return render('group/authz.html')
+
+
+    def _get_userobjectroles(self, group_id):
+        group = model.Group.get(group_id)
+        uors = model.Session.query(model.GroupRole).join('group').filter_by(name=group.name).all()
+        return uors
+
        
     def history(self, id):
         if 'diff' in request.params or 'selected1' in request.params:
@@ -419,8 +256,8 @@ class GroupController(BaseController):
                    'schema': self._form_to_db_schema()}
         data_dict = {'id': id}
         try:
-            c.group_dict = get.group_show(context, data_dict)
-            c.group_revisions = get.group_revision_list(context, data_dict)
+            c.group_dict = get_action('group_show')(context, data_dict)
+            c.group_revisions = get_action('group_revision_list')(context, data_dict)
             #TODO: remove
             # Still necessary for the authz check in group/layout.html
             c.group = context['group']
