@@ -2,11 +2,14 @@ import random
 import sys
 
 from pylons import cache, config
+from pylons.i18n import set_lang
 from genshi.template import NewTextTemplate
+import sqlalchemy.exc
 
 from ckan.authz import Authorizer
-from ckan.logic.action.get import current_package_list_with_resources
-from ckan.i18n import set_session_locale, set_lang
+from ckan.logic import NotAuthorized
+from ckan.logic import check_access, get_action
+from ckan.lib.i18n import set_session_locale, get_lang
 from ckan.lib.search import query_for, QueryOptions, SearchError
 from ckan.lib.cache import proxy_cache, get_cache_expires
 from ckan.lib.base import *
@@ -20,20 +23,32 @@ class HomeController(BaseController):
 
     def __before__(self, action, **env):
         BaseController.__before__(self, action, **env)
-        if not self.authorizer.am_authorized(c, model.Action.SITE_READ, model.System):
+        try:
+            context = {'model':model,'user': c.user or c.author}
+            check_access('site_read',context)
+        except NotAuthorized:
             abort(401, _('Not authorized to see this page'))
+        except (sqlalchemy.exc.ProgrammingError,
+                sqlalchemy.exc.OperationalError), e:
+            # postgres and sqlite errors for missing tables
+            msg = str(e)
+            if ('relation' in msg and 'does not exist' in msg) or \
+                   ('no such table' in msg) :
+                # table missing, major database problem
+                abort(503, _('This site is currently off-line. Database is not initialised.'))
+                # TODO: send an email to the admin person (#1285)
+            else:
+                raise
+            
 
     @staticmethod
-    def _home_cache_key(latest_revision_id=None):
-        '''Calculate the etag cache key for the home page. If you have
-        the latest revision id then supply it as a param.'''
-        if not latest_revision_id:
-            latest_revision_id = model.repo.youngest_revision().id
+    def _home_cache_key():
+        '''Calculate the etag cache key for the home page.'''
+        # a change to the data means the group package amounts may change
+        latest_revision_id = model.repo.youngest_revision().id
         user_name = c.user
-        if latest_revision_id:
-            cache_key = str(hash((latest_revision_id, user_name)))
-        else:
-            cache_key = 'fresh-install'
+        language = get_lang()
+        cache_key = str(hash((user_name, language, latest_revision_id)))
         return cache_key
 
     @proxy_cache(expires=cache_expires)
@@ -41,16 +56,17 @@ class HomeController(BaseController):
         cache_key = self._home_cache_key()
         etag_cache(cache_key)
 
-        query = query_for(model.Package)
-        query.run(query='*:*', facet_by=g.facets,
-                  limit=0, offset=0, username=c.user,
-                  order_by=None)
-        c.facets = query.facets
-        c.fields = []
-        c.package_count = query.count
-        c.latest_packages = current_package_list_with_resources({'model': model,
-                                                                 'user': c.user},
-                                                                 {'limit': 5})      
+        try:
+            query = query_for(model.Package)
+            query.run({'q': '*:*', 'facet.field': g.facets})
+            c.package_count = query.count
+            c.facets = query.facets # used by the 'tag cloud' recipe
+            q = model.Session.query(model.Group).filter_by(state='active')
+            c.groups = sorted(q.all(), key=lambda g: len(g.packages), reverse=True)[:6]
+        except SearchError, se:
+            c.package_count = 0
+            c.groups = []
+
         return render('home/index.html', cache_key=cache_key,
                       cache_expire=cache_expires)
 
@@ -60,11 +76,6 @@ class HomeController(BaseController):
     def about(self):
         return render('home/about.html', cache_expire=cache_expires)
         
-    def language(self):
-        response.content_type = 'text/javascript'
-        return render('home/language.js', cache_expire=cache_expires,
-                      method='text', loader_class=NewTextTemplate)
-    
     def locale(self): 
         locale = request.params.get('locale')
         if locale is not None:
@@ -74,6 +85,11 @@ class HomeController(BaseController):
                 abort(400, _('Invalid language specified'))
             try:
                 set_lang(locale)
+                # NOTE: When translating this string, substitute the word
+                # 'English' for the language being translated into.
+                # We do it this way because some Babel locales don't contain
+                # a display_name!
+                # e.g. babel.Locale.parse('no').get_display_name() returns None
                 h.flash_notice(_("Language has been set to: English"))
             except:
                 h.flash_notice("Language has been set to: English")
@@ -97,3 +113,7 @@ class HomeController(BaseController):
                 cache_.clear()
             return 'Cleared caches: %s' % ', '.join(wui_caches)
 
+    def cors_options(self, url=None):
+        # just return 200 OK and empty data
+        return ''
+
