@@ -9,12 +9,10 @@ from ckan.lib import mailer
 from ckan.authz import Authorizer
 from ckan.lib.navl.dictization_functions import DataError, unflatten
 from ckan.logic import NotFound, NotAuthorized, ValidationError
+from ckan.logic import check_access, get_action
 from ckan.logic import tuplize_dict, clean_dict, parse_params
 from ckan.logic.schema import user_new_form_schema, user_edit_form_schema 
-
-import ckan.logic.action.get as get
-import ckan.logic.action.create as create
-import ckan.logic.action.update as update
+from ckan.lib.captcha import check_recaptcha, CaptchaError
 
 log = logging.getLogger(__name__)
 
@@ -23,6 +21,15 @@ def login_form():
 
 class UserController(BaseController):
 
+    def __before__(self, action, **env):
+        BaseController.__before__(self, action, **env)
+        try:
+            context = {'model':model,'user': c.user or c.author}
+            check_access('site_read',context)
+        except NotAuthorized:
+            if c.action not in ('login','request_reset','perform_reset',):
+                abort(401, _('Not authorized to see this page'))
+
     ## hooks for subclasses 
     new_user_form = 'user/new_user_form.html'
     edit_user_form = 'user/edit_user_form.html'
@@ -49,40 +56,46 @@ class UserController(BaseController):
     def index(self):
         LIMIT = 20
 
-        if not self.authorizer.am_authorized(c, model.Action.USER_READ, model.System):
-            abort(401, _('Not authorized to see this page'))
-
         page = int(request.params.get('page', 1))
         c.q  = request.params.get('q', '')
         c.order_by = request.params.get('order_by', 'name')
 
         context = {'model': model,
-                   'user': c.user or c.author}
+                   'user': c.user or c.author,
+                   'return_query': True}
 
         data_dict = {'q':c.q,
                      'order_by':c.order_by}
+        try:
+            check_access('user_list',context, data_dict)
+        except NotAuthorized:
+            abort(401, _('Not authorized to see this page'))
 
-        users_list = get.user_list(context,data_dict)
+        users_list = get_action('user_list')(context,data_dict)
 
         c.page = h.Page(
             collection=users_list,
             page=page,
-            item_count=len(users_list),
+            item_count=users_list.count(),
             items_per_page=LIMIT
             )
         return render('user/list.html')
 
     def read(self, id=None):
-        if not self.authorizer.am_authorized(c, model.Action.USER_READ, model.System):
-            abort(401, _('Not authorized to see this page'))
 
         context = {'model': model,
                    'user': c.user or c.author}
 
         data_dict = {'id':id,
                      'user_obj':c.userobj}
+
         try:
-            user_dict = get.user_show(context,data_dict)
+            check_access('user_show',context, data_dict)
+        except NotAuthorized:
+            abort(401, _('Not authorized to see this page'))
+
+        try:
+            user_dict = get_action('user_show')(context,data_dict)
         except NotFound:
             h.redirect_to(controller='user', action='login', id=None)
 
@@ -110,8 +123,9 @@ class UserController(BaseController):
                    'schema': self._new_form_to_db_schema(),
                    'save': 'save' in request.params}
 
-        auth_for_create = Authorizer().am_authorized(c, model.Action.USER_CREATE, model.System())
-        if not auth_for_create:
+        try:
+            check_access('user_create',context)
+        except NotAuthorized:
             abort(401, _('Unauthorized to create a user'))
 
         if context['save'] and not data:
@@ -131,13 +145,18 @@ class UserController(BaseController):
             data_dict = clean_dict(unflatten(
                 tuplize_dict(parse_params(request.params))))
             context['message'] = data_dict.get('log_message', '')
-            user = create.user_create(context, data_dict)
+            check_recaptcha(request)
+            user = get_action('user_create')(context, data_dict)
         except NotAuthorized:
             abort(401, _('Unauthorized to create user %s') % '')
         except NotFound, e:
             abort(404, _('User not found'))
         except DataError:
             abort(400, _(u'Integrity Error'))
+        except CaptchaError:
+            error_msg = _(u'Bad Captcha. Please try again.')
+            h.flash_error(error_msg)
+            return self.new(data_dict)
         except ValidationError, e:
             errors = e.error_dict
             error_summary = e.error_summary
@@ -151,7 +170,6 @@ class UserController(BaseController):
     def edit(self, id=None, data=None, errors=None, error_summary=None):
         context = {'model': model, 'session': model.Session,
                    'user': c.user or c.author,
-                   'preview': 'preview' in request.params,
                    'save': 'save' in request.params,
                    'schema': self._edit_form_to_db_schema(),
                    }
@@ -162,11 +180,11 @@ class UserController(BaseController):
                 abort(400, _('No user specified'))
         data_dict = {'id': id}
 
-        if (context['save'] or context['preview']) and not data:
+        if (context['save']) and not data:
             return self._save_edit(id, context)
 
         try:
-            old_data = get.user_show(context, data_dict)
+            old_data = get_action('user_show')(context, data_dict)
 
             schema = self._db_to_edit_form_schema()
             if schema:
@@ -192,6 +210,7 @@ class UserController(BaseController):
 
         self._setup_template_variables(context)
 
+        c.is_myself = True
         c.form = render(self.edit_user_form, extra_vars=vars)
 
         return render('user/edit.html')
@@ -202,18 +221,9 @@ class UserController(BaseController):
                 tuplize_dict(parse_params(request.params))))
             context['message'] = data_dict.get('log_message', '')
             data_dict['id'] = id
-            user = update.user_update(context, data_dict)
-
-            if context['preview']:
-                about = request.params.getone('about')
-                c.preview = self._format_about(about)
-                c.user_about = about
-                c.full_name = request.params.get('fullname','')
-                c.email = request.params.getone('email')
-
-                return self.edit(id, data_dict)
-
-            h.redirect_to(controller='user', action='read', id=user['id'])
+            user = get_action('user_update')(context, data_dict)
+            h.flash_success(_('Profile updated'))
+            h.redirect_to(controller='user', action='read', id=user['name'])
         except NotAuthorized:
             abort(401, _('Unauthorized to edit user %s') % id)
         except NotFound, e:
@@ -238,12 +248,15 @@ class UserController(BaseController):
 
             data_dict = {'id':c.user}
 
-            user_dict = get.user_show(context,data_dict)
+            user_dict = get_action('user_show')(context,data_dict)
 
-            response.set_cookie("ckan_user", user_dict['name'])
-            response.set_cookie("ckan_display_name", user_dict['display_name'])
-            response.set_cookie("ckan_apikey", user_dict['apikey'])
-            h.flash_success(_("Welcome back, %s") % user_dict['display_name'])
+            # Max age of cookies: 50 years. Matches time set in templates/user/login.html
+            cookie_timeout=50*365*24*60*60
+
+            response.set_cookie("ckan_user", user_dict['name'], max_age=cookie_timeout)
+            response.set_cookie("ckan_display_name", user_dict['display_name'], max_age=cookie_timeout)
+            response.set_cookie("ckan_apikey", user_dict['apikey'], max_age=cookie_timeout)
+            h.flash_success(_("%s is now logged in") % user_dict['display_name'])
             return self.me()
         else:
             h.flash_error('Login failed. Bad username or password.')
@@ -266,7 +279,7 @@ class UserController(BaseController):
             data_dict = {'id':id}
             user_obj = None
             try:
-                user_dict = get.user_show(context,data_dict)
+                user_dict = get_action('user_show')(context,data_dict)
                 user_obj = context['user_obj']
             except NotFound:
                 # Try searching the user
@@ -274,13 +287,13 @@ class UserController(BaseController):
                 data_dict['q'] = id
 
                 if id and len(id) > 2:
-                    user_list = get.user_list(context,data_dict)
+                    user_list = get_action('user_list')(context,data_dict)
                     if len(user_list) == 1:
                         # This is ugly, but we need the user object for the mailer,
                         # and user_list does not return them
                         del data_dict['q']
                         data_dict['id'] = user_list[0]['id']
-                        user_dict = get.user_show(context,data_dict)
+                        user_dict = get_action('user_show')(context,data_dict)
                         user_obj = context['user_obj']
                     elif len(user_list) > 1:
                         h.flash_error(_('"%s" matched several users') % (id))
@@ -305,7 +318,7 @@ class UserController(BaseController):
         data_dict = {'id':id}
 
         try:
-            user_dict = get.user_show(context,data_dict)
+            user_dict = get_action('user_show')(context,data_dict)
             user_obj = context['user_obj']
         except NotFound, e:
             abort(404, _('User not found'))
@@ -321,7 +334,7 @@ class UserController(BaseController):
                 new_password = self._get_form_password()
                 user_dict['password'] = new_password
                 user_dict['reset_key'] = c.reset_key
-                user = update.user_update(context, user_dict)
+                user = get_action('user_update')(context, user_dict)
 
                 h.flash_success(_("Your password has been reset."))
                 redirect('/')
