@@ -28,10 +28,14 @@ class Globals(object):
         # has been setup in load_environment():
         self.site_id = config.get('ckan.site_id')
 
+        self.template_head_end = config.get('ckan.template_head_end', '')
         self.template_footer_end = config.get('ckan.template_footer_end', '')
-        
+
         # hide these extras fields on package read
         self.package_hide_extras = config.get('package_hide_extras', '').split()
 
         self.openid_enabled = asbool(config.get('openid_enabled', 'true'))
+
+        self.recaptcha_publickey = config.get('ckan.recaptcha.publickey', '')
+        self.recaptcha_privatekey = config.get('ckan.recaptcha.privatekey', '')
         
