@@ -38,9 +38,7 @@ class CkanCommand(paste.script.command.Command):
             msg = 'No config file supplied'
             raise self.BadCommand(msg)
         self.filename = os.path.abspath(self.options.config)
-        try:
-            fileConfig(self.filename)
-        except Exception: pass
+        fileConfig(self.filename)
         conf = appconfig('config:' + self.filename)
         load_environment(conf.global_conf, conf.local_conf)
 
@@ -63,11 +61,13 @@ class ManageDb(CkanCommand):
     db clean
     db upgrade [{version no.}] # Data migrate
     db dump {file-path} # dump to a pg_dump file
-    db dump-rdf {package-name} {file-path}
+    db dump-rdf {dataset-name} {file-path}
     db simple-dump-csv {file-path}
     db simple-dump-json {file-path}
     db send-rdf {talis-store} {username} {password}
     db load {file-path} # load a pg_dump from a file
+    db load-only {file-path} # load a pg_dump from a file but don\'t do
+                             # the schema upgrade or search indexing
     db create-from-model # create database from the model (indexes not made)
     '''
     summary = __doc__.split('\n')[0]
@@ -81,6 +81,7 @@ class ManageDb(CkanCommand):
         
         self._load_config()
         from ckan import model
+        import ckan.lib.search as search
 
         cmd = self.args[0]
         if cmd == 'init':
@@ -89,6 +90,7 @@ class ManageDb(CkanCommand):
                 print 'Initialising DB: SUCCESS'
         elif cmd == 'clean' or cmd == 'drop':
             model.repo.clean_db()
+            search.clear()
             if self.verbose:
                 print 'Cleaning DB: SUCCESS'
         elif cmd == 'upgrade':
@@ -96,8 +98,12 @@ class ManageDb(CkanCommand):
                 model.repo.upgrade_db(self.args[1])
             else:
                 model.repo.upgrade_db()
-        elif cmd == 'dump' or cmd == 'load':
-            self.dump_or_load(cmd)
+        elif cmd == 'dump':
+            self.dump(cmd)
+        elif cmd == 'load':
+            self.load(cmd)
+        elif cmd == 'load-only':
+            self.load(cmd, only_load=True)
         elif cmd == 'simple-dump-csv':
             self.simple_dump_csv(cmd)
         elif cmd == 'simple-dump-json':
@@ -157,23 +163,38 @@ class ManageDb(CkanCommand):
 
     def _run_cmd(self, command_line):
         import subprocess    
-        subprocess.call(command_line, shell=True)
+        retcode = subprocess.call(command_line, shell=True)
+        if retcode != 0:
+            raise SystemError('Command exited with errorcode: %i' % retcode)
 
-    def dump_or_load(self, cmd):
+    def dump(self, cmd):
         if len(self.args) < 2:
             print 'Need pg_dump filepath'
             return
         dump_path = self.args[1]
 
         psql_cmd = self._get_psql_cmd() + ' -f %s'
-        if cmd == 'load':
-            pg_cmd = self._postgres_load(dump_path)
-        elif cmd == 'dump':
-            pg_cmd = self._postgres_dump(dump_path)
+        pg_cmd = self._postgres_dump(dump_path)
+
+    def load(self, cmd, only_load=False):
+        if len(self.args) < 2:
+            print 'Need pg_dump filepath'
+            return
+        dump_path = self.args[1]
+
+        psql_cmd = self._get_psql_cmd() + ' -f %s'
+        pg_cmd = self._postgres_load(dump_path)
+        if not only_load:
+            print 'Upgrading DB'
+            from ckan import model
+            model.repo.upgrade_db()
+            
+            print 'Rebuilding search index'
+            import ckan.lib.search
+            ckan.lib.search.rebuild()
         else:
-            print 'Unknown command', cmd
-        if cmd == 'load':
-            print 'Now remember you have to call \'db upgrade\'.'
+            print 'Now remember you have to call \'db upgrade\' and then \'search-index rebuild\'.'
+        print 'Done'
 
     def simple_dump_csv(self, cmd):
         from ckan import model
@@ -199,7 +220,7 @@ class ManageDb(CkanCommand):
 
     def dump_rdf(self, cmd):
         if len(self.args) < 3:
-            print 'Need package name and rdf file path'
+            print 'Need dataset name and rdf file path'
             return
         package_name = self.args[1]
         rdf_path = self.args[2]
@@ -207,7 +228,7 @@ class ManageDb(CkanCommand):
         import ckan.lib.rdf as rdf
         pkg = model.Package.by_name(unicode(package_name))
         if not pkg:
-            print 'Package name "%s" does not exist' % package_name
+            print 'Dataset name "%s" does not exist' % package_name
             return
         rdf = rdf.RdfExporter().export_package(pkg)
         f = open(rdf_path, 'w')
@@ -227,12 +248,13 @@ class ManageDb(CkanCommand):
 
 
 class SearchIndexCommand(CkanCommand):
-    '''Creates a search index for all packages
+    '''Creates a search index for all datasets
 
     Usage:
-      search-index rebuild                 - indexes all packages (default)
+      search-index rebuild                 - indexes all packages
       search-index check                   - checks for packages not indexed
       search-index show {package-name}     - shows index of a package
+      search-index clear                   - clears the search index for this ckan instance
     '''
 
     summary = __doc__.split('\n')[0]
@@ -242,14 +264,14 @@ class SearchIndexCommand(CkanCommand):
 
     def command(self):
         self._load_config()
-        from ckan.lib.search import rebuild, check, show
+        from ckan.lib.search import rebuild, check, show, clear
 
         if not self.args:
-            # default to run
-            cmd = 'rebuild'
-        else:
-            cmd = self.args[0]
-        
+            # default to printing help
+            print self.usage
+            return
+
+        cmd = self.args[0]        
         if cmd == 'rebuild':
             rebuild()
         elif cmd == 'check':
@@ -259,13 +281,15 @@ class SearchIndexCommand(CkanCommand):
                 import pdb; pdb.set_trace()
                 self.args
             show(self.args[1])
+        elif cmd == 'clear':
+            clear()
         else:
             print 'Command %s not recognized' % cmd
 
 class Notification(CkanCommand):
     '''Send out modification notifications.
     
-    In "replay" mode, an update signal is sent for each package in the database.
+    In "replay" mode, an update signal is sent for each dataset in the database.
 
     Usage:
       notify replay                        - send out modification signals
@@ -716,39 +740,39 @@ class Changes(CkanCommand):
         except UncommittedChangesException, inst:
             print "There are uncommitted revisions (run 'changes commit')."
             sys.exit(1)
-        print ", ".join(["%s %s packages" % (key, len(val)) for (key, val) in report.items()])
+        print ", ".join(["%s %s datasets" % (key, len(val)) for (key, val) in report.items()])
         if report['created']:
             print ""
-            print "The following packages have been created:"
+            print "The following datasets have been created:"
             names = []
             for entity in report['created']:
                 if not entity:
                     continue
                 if entity.name in names:
                     continue
-                print "package:    %s" % entity.name
+                print "dataset:    %s" % entity.name
                 names.append(entity.name)
         if report['updated']:
             print ""
-            print "The following packages have been updated:"
+            print "The following datasets have been updated:"
             names = []
             for entity in report['updated']:
                 if not entity:
                     continue
                 if entity.name in names:
                     continue
-                print "package:    %s" % entity.name
+                print "dataset:    %s" % entity.name
                 names.append(entity.name)
         if report['deleted']:
             print ""
-            print "The following packages have been deleted:"
+            print "The following datasets have been deleted:"
             names = []
             for entity in report['deleted']:
                 if not entity:
                     continue
                 if entity.name in names:
                     continue
-                print "package:    %s" % entity.name
+                print "dataset:    %s" % entity.name
                 names.append(entity.name)
 
     def moderate_changeset_apply(self, changeset):
