@@ -1,5 +1,6 @@
 import cli
 from collections import defaultdict
+import datetime
 
 class CreateTestData(cli.CkanCommand):
     '''Create test data in the database.
@@ -137,7 +138,12 @@ class CreateTestData(cli.CkanCommand):
                     elif attr == 'resources':
                         assert isinstance(val, (list, tuple))
                         for res_dict in val:
-                            non_extras = dict([(str(k), unicode(v)) for k, v in res_dict.items() if k != 'extras'])
+                            non_extras = {}
+                            for k, v in res_dict.items():
+                                if k != 'extras':
+                                    if not isinstance(v, datetime.datetime):
+                                        v = unicode(v)
+                                    non_extras[str(k)] = v
                             extras = dict([(str(k), unicode(v)) for k, v in res_dict.get('extras', {}).items()])
                             pkg.add_resource(extras=extras, **non_extras)
                     elif attr == 'tags':
@@ -320,7 +326,7 @@ class CreateTestData(cli.CkanCommand):
             format=u'plain text',
             description=u'Full text. Needs escaping: " Umlaut: \xfc',
             hash=u'abc123',
-            extras={'size': u'123'},
+            extras={'size_extra': u'123'},
             **configured_extras[0]
             )
         pr2 = model.Resource(
@@ -328,7 +334,7 @@ class CreateTestData(cli.CkanCommand):
             format=u'json',
             description=u'Index of the novel',
             hash=u'def456',
-            extras={'size': u'345'},
+            extras={'size_extra': u'345'},
             **configured_extras[1]
             )
         model.Session.add(pr1)
@@ -525,6 +531,13 @@ search_items = [{'name':'gils',
               'title':'Government Information Locator Service',
               'url':'',
               'tags':'registry  country-usa  government  federal  gov  workshop-20081101 penguin',
+              'resources':[{'url':'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000859/SFR17_2009_tables.xls',
+                          'format':'XLS',
+                          'last_modified': datetime.datetime(2005,10,01),
+                          'description':'December 2009 | http://www.statistics.gov.uk/hub/id/119-36345'},
+                          {'url':'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000860/SFR17_2009_key.doc',
+                          'format':'DOC',
+                          'description':'http://www.statistics.gov.uk/hub/id/119-34565'}],
               'groups':'ukgov test1 test2 penguin',
               'license':'gpl-3.0',
               'notes':u'''From <http://www.gpoaccess.gov/gils/about.html>
