@@ -19,7 +19,9 @@ from pylons.decorators.cache import beaker_cache
 from routes import url_for, redirect_to
 from alphabet_paginate import AlphaPage
 from lxml.html import fromstring
-from ckan.i18n import get_available_locales
+from i18n import get_available_locales
+
+
 
 try:
     from collections import OrderedDict # from python 2.7
@@ -42,9 +44,10 @@ class Message(object):
     * ``category``: the category specified when the message was created.
     """
 
-    def __init__(self, category, message):
+    def __init__(self, category, message, allow_html):
         self.category=category
         self.message=message
+        self.allow_html=allow_html
 
     def __str__(self):
         return self.message
@@ -52,7 +55,10 @@ class Message(object):
     __unicode__ = __str__
 
     def __html__(self):
-        return escape(self.message)
+        if self.allow_html: 
+            return self.message
+        else:
+            return escape(self.message)
 
 class _Flash(object):
     
@@ -71,14 +77,14 @@ class _Flash(object):
         if self.categories and self.default_category not in self.categories:
             raise ValueError("unrecognized default category %r" % (self.default_category,))
 
-    def __call__(self, message, category=None, ignore_duplicate=False):
+    def __call__(self, message, category=None, ignore_duplicate=False, allow_html=False):
         if not category:
             category = self.default_category
         elif self.categories and category not in self.categories:
             raise ValueError("unrecognized category %r" % (category,))
         # Don't store Message objects in the session, to avoid unpickling
         # errors in edge cases.
-        new_message_tuple = (category, message)
+        new_message_tuple = (category, message, allow_html)
         from pylons import session
         messages = session.setdefault(self.session_key, [])
         # ``messages`` is a mutable list, so changes to the local variable are
@@ -99,16 +105,23 @@ class _Flash(object):
         session.save()
         return [Message(*m) for m in messages]
 
+    def are_there_messages(self):
+        from pylons import session
+        return bool(session.get(self.session_key))
+
 _flash = _Flash()
 
-def flash_notice(message): 
-    _flash(message, category='notice')
+def flash_notice(message, allow_html=False): 
+    _flash(message, category='notice', allow_html=allow_html)
 
-def flash_error(message): 
-    _flash(message, category='error')
+def flash_error(message, allow_html=False): 
+    _flash(message, category='error', allow_html=allow_html)
 
-def flash_success(message): 
-    _flash(message, category='success')
+def flash_success(message, allow_html=False): 
+    _flash(message, category='success', allow_html=allow_html)
+
+def are_there_flash_messages():
+    return _flash.are_there_messages()
 
 # FIXME: shouldn't have to pass the c object in to this.
 def nav_link(c, text, controller, **kwargs):
@@ -147,12 +160,29 @@ def facet_title(name):
     return config.get('search.facets.%s.title' % name, name.capitalize())
 
 def am_authorized(c, action, domain_object=None):
+    ''' Deprecated. Please use check_access instead'''
     from ckan.authz import Authorizer
     if domain_object is None:
         from ckan import model
         domain_object = model.System()
     return Authorizer.am_authorized(c, action, domain_object)
 
+def check_access(action,data_dict=None):
+    from ckan import model
+    from ckan.lib.base import c
+    from ckan.logic import check_access as check_access_logic,NotAuthorized
+
+    context = {'model': model,
+                'user': c.user or c.author}
+
+    try:
+        check_access_logic(action,context,data_dict)
+        authorized = True
+    except NotAuthorized:
+        authorized = False
+
+    return authorized
+
 def linked_user(user, maxlength=0):
     from ckan import model
     from urllib import quote
@@ -241,3 +271,7 @@ def date_str_to_datetime(date_str):
 def time_ago_in_words_from_str(date_str, granularity='month'):
     return date.time_ago_in_words(date_str_to_datetime(date_str), granularity=granularity)
 
+def button_attr(enable, type='primary'):
+    if enable:
+        return 'class="pretty-button %s"' % type
+    return 'disabled class="pretty-button disabled"'
