@@ -1,12 +1,12 @@
 from sqlalchemy.sql import select
-from sqlalchemy import or_, func, desc
+from sqlalchemy import or_, and_, func, desc, case
 
-from ckan.logic import NotFound, check_access
+from ckan.logic import NotFound
+from ckan.logic import check_access
 from ckan.plugins import (PluginImplementations,
                           IGroupController,
                           IPackageController)
-import ckan.authz
-
+from ckan.authz import Authorizer
 from ckan.lib.dictization import table_dictize
 from ckan.lib.dictization.model_dictize import (package_dictize,
                                                 resource_list_dictize,
@@ -21,16 +21,29 @@ from ckan.lib.dictization.model_dictize import (package_to_api1,
                                                 group_to_api2,
                                                 tag_to_api1,
                                                 tag_to_api2)
-from ckan.lib.search import query_for
+from ckan.lib.search import query_for, SearchError
+import logging
+
+log = logging.getLogger('ckan.logic')
+
+def site_read(context,data_dict=None):
+    check_access('site_read',context,data_dict)
+    return True
 
 def package_list(context, data_dict):
-    '''Lists the package by name'''
+    '''Lists packages by name or id'''
+
     model = context["model"]
     user = context["user"]
     api = context.get("api_version", '1')
     ref_package_by = 'id' if api == '2' else 'name'
+    
+    check_access('package_list', context, data_dict)
+
+    query = model.Session.query(model.PackageRevision)
+    query = query.filter(model.PackageRevision.state=='active')
+    query = query.filter(model.PackageRevision.current==True)
 
-    query = ckan.authz.Authorizer().authorized_query(user, model.Package)
     packages = query.all()
     return [getattr(p, ref_package_by) for p in packages]
 
@@ -39,23 +52,25 @@ def current_package_list_with_resources(context, data_dict):
     user = context["user"]
     limit = data_dict.get("limit")
 
-    q = ckan.authz.Authorizer().authorized_query(user, model.PackageRevision)
-    q = q.filter(model.PackageRevision.state=='active')
-    q = q.filter(model.PackageRevision.current==True)
+    check_access('current_package_list_with_resources', context, data_dict)
 
-    q = q.order_by(model.package_revision_table.c.revision_timestamp.desc())
+    query = model.Session.query(model.PackageRevision)
+    query = query.filter(model.PackageRevision.state=='active')
+    query = query.filter(model.PackageRevision.current==True)
+
+    query = query.order_by(model.package_revision_table.c.revision_timestamp.desc())
     if limit:
-        q = q.limit(limit)
-    pack_rev = q.all()
+        query = query.limit(limit)
+    pack_rev = query.all()
     package_list = []
     for package in pack_rev:
         result_dict = table_dictize(package, context)
         res_rev = model.resource_revision_table
         resource_group = model.resource_group_table
-        q = select([res_rev], from_obj = res_rev.join(resource_group, 
+        query = select([res_rev], from_obj = res_rev.join(resource_group,
                    resource_group.c.id == res_rev.c.resource_group_id))
-        q = q.where(resource_group.c.package_id == package.id)
-        result = q.where(res_rev.c.current == True).execute()
+        query = query.where(resource_group.c.package_id == package.id)
+        result = query.where(res_rev.c.current == True).execute()
         result_dict["resources"] = resource_list_dictize(result, context)
         license_id = result_dict['license_id']
         if license_id:
@@ -64,7 +79,7 @@ def current_package_list_with_resources(context, data_dict):
                 result_dict['isopen'] = isopen
             except KeyError:
                 # TODO: create a log message this error?
-                result_dict['isopen'] = False 
+                result_dict['isopen'] = False
         else:
             result_dict['isopen'] = False
         package_list.append(result_dict)
@@ -72,7 +87,10 @@ def current_package_list_with_resources(context, data_dict):
 
 def revision_list(context, data_dict):
 
-    model = context["model"]
+    model = context['model']
+
+    check_access('revision_list', context, data_dict)
+
     revs = model.Session.query(model.Revision).all()
     return [rev.id for rev in revs]
 
@@ -82,7 +100,8 @@ def package_revision_list(context, data_dict):
     pkg = model.Package.get(id)
     if pkg is None:
         raise NotFound
-    check_access(pkg, model.Action.READ, context)
+
+    check_access('package_revision_list',context, data_dict)
 
     revision_dicts = []
     for revision, object_revisions in pkg.all_related_revisions:
@@ -100,41 +119,50 @@ def group_list(context, data_dict):
     ref_group_by = 'id' if api == '2' else 'name';
 
     all_fields = data_dict.get('all_fields',None)
+   
+    check_access('group_list',context, data_dict)
 
-    query = ckan.authz.Authorizer().authorized_query(user, model.Group)
+    # We need Groups for group_list_dictize
+    query = model.Session.query(model.Group).join(model.GroupRevision)
+    query = query.filter(model.GroupRevision.state=='active')
+    query = query.filter(model.GroupRevision.current==True)
     query = query.order_by(model.Group.name.asc())
     query = query.order_by(model.Group.title.asc())
 
+
     groups = query.all()
 
     if not all_fields:
         group_list = [getattr(p, ref_group_by) for p in groups]
     else:
         group_list = group_list_dictize(groups,context)
-    
+
     return group_list
 
 def group_list_authz(context, data_dict):
+    '''
+    Returns a list of groups which the user is allowed to edit
+
+    If 'available_only' is specified, the existing groups in the package are
+    removed.
+
+    '''
     model = context['model']
     user = context['user']
-    pkg = context.get('package')
+    available_only = data_dict.get('available_only',False)
 
-    query = ckan.authz.Authorizer().authorized_query(user, model.Group, model.Action.EDIT)
+    check_access('group_list_authz',context, data_dict)
+
+    from ckan.authz import Authorizer
+    query = Authorizer().authorized_query(user, model.Group, model.Action.EDIT)
     groups = set(query.all())
-    return dict((group.id, group.name) for group in groups)
+    
+    if available_only:
+        package = context.get('package')
+        if package:
+            groups = groups - set(package.groups)
 
-def group_list_available(context, data_dict):
-    model = context['model']
-    user = context['user']
-    pkg = context.get('package')
-
-    query = ckan.authz.Authorizer().authorized_query(user, model.Group, model.Action.EDIT)
-    groups = set(query.all())
-
-    if pkg:
-        groups = groups - set(pkg.groups)
-
-    return [(group.id, group.name) for group in groups]
+    return [{'id':group.id,'name':group.name} for group in groups]
 
 def group_revision_list(context, data_dict):
     model = context['model']
@@ -142,7 +170,8 @@ def group_revision_list(context, data_dict):
     group = model.Group.get(id)
     if group is None:
         raise NotFound
-    check_access(group, model.Action.READ, context)
+
+    check_access('group_revision_list',context, data_dict)
 
     revision_dicts = []
     for revision, object_revisions in group.all_related_revisions:
@@ -153,6 +182,9 @@ def group_revision_list(context, data_dict):
 
 def licence_list(context, data_dict):
     model = context["model"]
+
+    check_access('licence_list',context, data_dict)
+
     license_register = model.Package.get_license_register()
     licenses = license_register.values()
     licences = [l.as_dict() for l in licenses]
@@ -166,13 +198,15 @@ def tag_list(context, data_dict):
 
     all_fields = data_dict.get('all_fields',None)
 
+    check_access('tag_list',context, data_dict)
+
     q = data_dict.get('q','')
     if q:
         limit = data_dict.get('limit',25)
         offset = data_dict.get('offset',0)
         return_objects = data_dict.get('return_objects',True)
 
-        query = query_for(model.Tag, backend='sql')
+        query = query_for(model.Tag)
         query.run(query=q,
                   limit=limit,
                   offset=offset,
@@ -180,8 +214,8 @@ def tag_list(context, data_dict):
                   username=user)
         tags = query.results
     else:
-        tags = model.Session.query(model.Tag).all() 
-    
+        tags = model.Session.query(model.Tag).all()
+
     tag_list = []
     if all_fields:
         for tag in tags:
@@ -197,23 +231,51 @@ def user_list(context, data_dict):
     model = context['model']
     user = context['user']
 
+    check_access('user_list',context, data_dict)
+
     q = data_dict.get('q','')
     order_by = data_dict.get('order_by','name')
 
-    query = model.Session.query(model.User, func.count(model.User.id))
+    query = model.Session.query(
+        model.User,
+        model.User.name.label('name'),
+        model.User.fullname.label('fullname'),
+        model.User.about.label('about'),
+        model.User.about.label('email'),
+        model.User.created.label('created'),
+        select([func.count(model.Revision.id)], or_(
+                model.Revision.author==model.User.name,
+                model.Revision.author==model.User.openid
+                )
+        ).label('number_of_edits'),
+        select([func.count(model.UserObjectRole.id)], and_(
+            model.UserObjectRole.user_id==model.User.id,
+            model.UserObjectRole.context=='Package',
+            model.UserObjectRole.role=='admin'
+            )
+        ).label('number_administered_packages')
+    )
+
     if q:
         query = model.User.search(q, query)
 
     if order_by == 'edits':
-        query = query.join((model.Revision, or_(
+        query = query.order_by(desc(
+            select([func.count(model.Revision.id)], or_(
                 model.Revision.author==model.User.name,
                 model.Revision.author==model.User.openid
-                )))
-        query = query.group_by(model.User)
-        query = query.order_by(desc(func.count(model.User.id)))
+                ))
+        ))
     else:
-        query = query.group_by(model.User)
-        query = query.order_by(model.User.name)
+        query = query.order_by(
+            case([(or_(model.User.fullname == None, model.User.fullname == ''),
+                   model.User.name)],
+                 else_=model.User.fullname)
+        )
+
+    ## hack for pagination
+    if context.get('return_query'):
+        return query
 
     users_list = []
 
@@ -247,15 +309,18 @@ def package_relationships_list(context, data_dict):
     if rel == 'relationships':
         rel = None
 
-    relationships = ckan.authz.Authorizer().\
+    check_access('package_relationships_list',context, data_dict)
+    
+    # TODO: How to handle this object level authz?
+    relationships = Authorizer().\
                     authorized_package_relationships(\
                     user, pkg1, pkg2, rel, model.Action.READ)
 
     if rel and not relationships:
         raise NotFound('Relationship "%s %s %s" not found.'
                                  % (id, rel, id2))
-    
-    relationship_dicts = [rel.as_dict(pkg1, ref_package_by=ref_package_by) 
+
+    relationship_dicts = [rel.as_dict(pkg1, ref_package_by=ref_package_by)
                           for rel in relationships]
 
     return relationship_dicts
@@ -272,7 +337,8 @@ def package_show(context, data_dict):
 
     if pkg is None:
         raise NotFound
-    check_access(pkg, model.Action.READ, context)
+
+    check_access('package_show',context, data_dict)
 
     package_dict = package_dictize(pkg, context)
 
@@ -309,7 +375,7 @@ def group_show(context, data_dict):
     if group is None:
         raise NotFound
 
-    check_access(group, model.Action.READ, context)
+    check_access('group_show',context, data_dict)
 
     group_dict = group_dictize(group, context)
 
@@ -332,6 +398,8 @@ def tag_show(context, data_dict):
     if tag is None:
         raise NotFound
 
+    check_access('tag_show',context, data_dict)
+
     tag_dict = tag_dictize(tag,context)
     extended_packages = []
     for package in tag_dict['packages']:
@@ -344,24 +412,32 @@ def tag_show(context, data_dict):
 def user_show(context, data_dict):
     '''Shows user details'''
     model = context['model']
+    user = context['user']
 
     id = data_dict.get('id',None)
     provided_user = data_dict.get('user_obj',None)
     if id:
-        user = model.User.get(id)
-        context['user_obj'] = user
-        if user is None:
+        user_obj = model.User.get(id)
+        context['user_obj'] = user_obj
+        if user_obj is None:
             raise NotFound
     elif provided_user:
-        context['user_obj'] = user = provided_user
+        context['user_obj'] = user_obj = provided_user
     else:
         raise NotFound
 
-    user_dict = user_dictize(user,context)
+    check_access('user_show',context, data_dict)
+
+    user_dict = user_dictize(user_obj,context)
+
+    if not (Authorizer().is_sysadmin(unicode(user)) or user == user_obj.name):
+        # If not sysadmin or the same user, strip sensible info
+        del user_dict['apikey']
+        del user_dict['reset_key']
 
     revisions_q = model.Session.query(model.Revision
-            ).filter_by(author=user.name)
-    
+            ).filter_by(author=user_obj.name)
+
     revisions_list = []
     for revision in revisions_q.limit(20).all():
         revision_dict = revision_show(context,{'id':revision.id})
@@ -374,6 +450,8 @@ def user_show(context, data_dict):
 
 def package_show_rest(context, data_dict):
 
+    check_access('package_show_rest',context, data_dict)
+
     package_show(context, data_dict)
 
     api = context.get('api_version') or '1'
@@ -388,6 +466,8 @@ def package_show_rest(context, data_dict):
 
 def group_show_rest(context, data_dict):
 
+    check_access('group_show_rest',context, data_dict)
+
     group_show(context, data_dict)
     api = context.get('api_version') or '1'
     group = context['group']
@@ -401,6 +481,8 @@ def group_show_rest(context, data_dict):
 
 def tag_show_rest(context, data_dict):
 
+    check_access('tag_show_rest',context, data_dict)
+
     tag_show(context, data_dict)
     api = context.get('api_version') or '1'
     tag = context['tag']
@@ -414,6 +496,7 @@ def tag_show_rest(context, data_dict):
 
 def package_autocomplete(context, data_dict):
     '''Returns packages containing the provided string'''
+
     model = context['model']
     session = context['session']
     user = context['user']
@@ -421,42 +504,73 @@ def package_autocomplete(context, data_dict):
 
     like_q = u"%s%%" % q
 
-    #TODO: Auth
-    pkg_query = ckan.authz.Authorizer().authorized_query(user, model.Package)
-    pkg_query = session.query(model.Package) \
-                    .filter(or_(model.Package.name.ilike(like_q),
-                                model.Package.title.ilike(like_q)))
-    pkg_query = pkg_query.limit(10)
+    check_access('package_autocomplete', context, data_dict)
+
+    query = model.Session.query(model.PackageRevision)
+    query = query.filter(model.PackageRevision.state=='active')
+    query = query.filter(model.PackageRevision.current==True)
+    query = query.filter(or_(model.PackageRevision.name.ilike(like_q),
+                                model.PackageRevision.title.ilike(like_q)))
+    query = query.limit(10)
 
     pkg_list = []
-    for package in pkg_query:
-        result_dict = table_dictize(package, context)
+    for package in query:
+        result_dict = {'name':package.name,'title':package.title}
         pkg_list.append(result_dict)
 
     return pkg_list
 
 def tag_autocomplete(context, data_dict):
     '''Returns tags containing the provided string'''
+
     model = context['model']
     session = context['session']
     user = context['user']
 
-    q = data_dict.get('q',None)
+    check_access('tag_autocomplete', context, data_dict)
+
+    q = data_dict.get('q', None)
     if not q:
         return []
 
     limit = data_dict.get('limit',10)
 
-    like_q = u"%s%%" % q
-
-    query = query_for('tag', backend='sql')
-    query.run(query=like_q,
+    query = query_for('tag')
+    query.run(query=q,
               return_objects=True,
               limit=10,
               username=user)
 
     return [tag.name for tag in query.results]
 
+def format_autocomplete(context, data_dict):
+    '''Returns formats containing the provided string'''
+    model = context['model']
+    session = context['session']
+    user = context['user']
+
+    check_access('format_autocomplete', context, data_dict)
+
+    q = data_dict.get('q', None)
+    if not q:
+        return []
+
+    limit = data_dict.get('limit', 5)
+    like_q = u'%' + q + u'%'
+
+    query = session.query(model.ResourceRevision.format,
+        func.count(model.ResourceRevision.format).label('total'))\
+        .filter(and_(
+            model.ResourceRevision.state == 'active',
+            model.ResourceRevision.current == True
+        ))\
+        .filter(model.ResourceRevision.format.ilike(like_q))\
+        .group_by(model.ResourceRevision.format)\
+        .order_by('total DESC')\
+        .limit(limit)
+
+    return [resource.format for resource in query]
+
 def user_autocomplete(context, data_dict):
     '''Returns users containing the provided string'''
     model = context['model']
@@ -466,6 +580,8 @@ def user_autocomplete(context, data_dict):
     if not q:
         return []
 
+    check_access('user_autocomplete', context, data_dict)
+
     limit = data_dict.get('limit',20)
 
     query = model.User.search(q).limit(limit)
@@ -485,31 +601,33 @@ def package_search(context, data_dict):
     session = context['session']
     user = context['user']
 
-    q=data_dict.get('q','')
-    fields=data_dict.get('fields',[])
-    facet_by=data_dict.get('facet_by',[])
-    limit=data_dict.get('limit',20)
-    offset=data_dict.get('offset',0)
-    return_objects=data_dict.get('return_objects',False)
-    filter_by_openness=data_dict.get('filter_by_openness',False)
-    filter_by_downloadable=data_dict.get('filter_by_downloadable',False)
+    check_access('package_search', context, data_dict)
+
+    # return a list of package ids
+    data_dict['fl'] = 'id'
 
     query = query_for(model.Package)
-    query.run(query=q,
-              fields=fields,
-              facet_by=facet_by,
-              limit=limit,
-              offset=offset,
-              return_objects=return_objects,
-              filter_by_openness=filter_by_openness,
-              filter_by_downloadable=filter_by_downloadable,
-              username=user)
-    
+    query.run(data_dict)
+
     results = []
     for package in query.results:
-        result_dict = table_dictize(package, context)
-        result_dict = _extend_package_dict(result_dict,context)
+        # get the package object
+        pkg_query = session.query(model.PackageRevision)\
+            .filter(model.PackageRevision.id == package)\
+            .filter(and_(
+                model.PackageRevision.state == u'active', 
+                model.PackageRevision.current == True
+            ))
+        pkg = pkg_query.first()
 
+        ## if the index has got a package that is not in ckan then
+        ## ignore it.
+        if not pkg:
+            log.warning('package %s in index but not in database' % package)
+            continue
+
+        result_dict = table_dictize(pkg, context)
+        result_dict = _extend_package_dict(result_dict,context)
         results.append(result_dict)
 
     return {
@@ -529,7 +647,7 @@ def _extend_package_dict(package_dict,context):
         package_dict['resources'] = resource_list_dictize(resources, context)
     else:
         package_dict['resources'] = []
-    license_id = package_dict['license_id']
+    license_id = package_dict.get('license_id')
     if license_id:
         try:
             isopen = model.Package.get_license_register()[license_id].isopen()
@@ -540,3 +658,84 @@ def _extend_package_dict(package_dict,context):
         package_dict['isopen'] = False
 
     return package_dict
+
+def resource_search(context, data_dict):
+    model = context['model']
+    session = context['session']
+
+    fields = data_dict['fields']
+    order_by = data_dict.get('order_by')
+    offset = data_dict.get('offset')
+    limit = data_dict.get('limit')
+
+    # TODO: should we check for user authentication first?
+    q = model.Session.query(model.Resource)
+    resource_fields = model.Resource.get_columns()
+
+    for field, terms in fields.items():
+        if isinstance(terms, basestring):
+            terms = terms.split()
+        if field not in resource_fields:
+            raise SearchError('Field "%s" not recognised in Resource search.' % field)
+        for term in terms:
+            model_attr = getattr(model.Resource, field)
+            if field == 'hash':                
+                q = q.filter(model_attr.ilike(unicode(term) + '%'))
+            elif field in model.Resource.get_extra_columns():
+                model_attr = getattr(model.Resource, 'extras')
+
+                like = or_(
+                    model_attr.ilike(u'''%%"%s": "%%%s%%",%%''' % (field, term)),
+                    model_attr.ilike(u'''%%"%s": "%%%s%%"}''' % (field, term))
+                )
+                q = q.filter(like)
+            else:
+                q = q.filter(model_attr.ilike('%' + unicode(term) + '%'))
+    
+    if order_by is not None:
+        if hasattr(model.Resource, order_by):
+            q = q.order_by(getattr(model.Resource, order_by))
+
+    count = q.count()
+    q = q.offset(offset)
+    q = q.limit(limit)
+    
+    results = []
+    for result in q:
+        if isinstance(result, tuple) and isinstance(result[0], model.DomainObject):
+            # This is the case for order_by rank due to the add_column.
+            results.append(result[0])
+        else:
+            results.append(result)
+
+    return {'count': count, 'results': results}
+
+def tag_search(context, data_dict):
+    model = context['model']
+    session = context['session']
+
+    query = data_dict.get('query')
+    terms = [query] if query else []
+
+    fields = data_dict.get('fields', {})
+    offset = data_dict.get('offset')
+    limit = data_dict.get('limit')
+
+    # TODO: should we check for user authentication first?
+    q = model.Session.query(model.Tag)
+    q = q.distinct().join(model.Tag.package_tags)
+    for field, value in fields.items():
+        if field in ('tag', 'tags'):
+            terms.append(value)
+
+    if not len(terms):
+        return
+
+    for term in terms:
+        q = q.filter(model.Tag.name.contains(term.lower()))
+
+    count = q.count()
+    q = q.offset(offset)
+    q = q.limit(limit)
+    results = [r for r in q]
+    return {'count': count, 'results': results}
