@@ -1,3 +1,5 @@
+import logging
+
 from vdm.sqlalchemy import State
 
 from sqlalchemy.orm import object_session
@@ -14,6 +16,7 @@ from ckan.model.resource import ResourceGroup, Resource
 from ckan.model.package_extra import PackageExtra
 from ckan.model.tag import PackageTag
 
+log = logging.getLogger(__name__)
 
 class DomainObjectModificationExtension(SingletonPlugin, ObserverNotifier):
     """
@@ -52,12 +55,10 @@ class DomainObjectModificationExtension(SingletonPlugin, ObserverNotifier):
                     related_packages = obj.related_packages()
                 except AttributeError:
                     continue
-                if 'pending' in obj.state:
-                    continue
                 # this is needed to sort out vdm bug where pkg.as_dict does not
                 # work when the package is deleted.
                 for package in related_packages:
-                    if package not in deleted | new:
+                    if package and package not in deleted | new:
                         changed_pkgs.add(package)
         for obj in changed_pkgs:
             self.notify(obj, DomainObjectOperation.changed)
@@ -65,4 +66,10 @@ class DomainObjectModificationExtension(SingletonPlugin, ObserverNotifier):
 
     def notify(self, entity, operation):
         for observer in self.observers:
-            observer.notify(entity, operation)
+            try:
+                observer.notify(entity, operation)
+            except Exception, ex:
+                log.exception(ex)
+                # We reraise all exceptions so they are obvious there
+                # is something wrong
+                raise
