@@ -1,3 +1,4 @@
+import re
 import json
 from pprint import pprint, pformat
 from nose.tools import assert_equal
@@ -23,9 +24,7 @@ class TestAction(WsgiAppCase):
     @classmethod
     def setup_class(self):
         CreateTestData.create()
-
         self.sysadmin_user = model.User.get('testsysadmin')
-
         self.normal_user = model.User.get('annafan')
 
     @classmethod
@@ -37,16 +36,45 @@ class TestAction(WsgiAppCase):
         res = self.app.post('/api/action/package_list', params=postparams)
         assert_dicts_equal_ignoring_ordering(
             json.loads(res.body),
-            {"help": "Lists the package by name",
+            {"help": "Lists packages by name or id",
              "success": True,
              "result": ["annakarenina", "warandpeace"]})
-        
+
+    def test_01_package_show(self):
+        anna_id = model.Package.by_name(u'annakarenina').id
+        postparams = '%s=1' % json.dumps({'id': anna_id})
+        res = self.app.post('/api/action/package_show', params=postparams)
+        res_dict = json.loads(res.body)
+        assert_equal(res_dict['success'], True)
+        assert_equal(res_dict['help'], None)
+        pkg = res_dict['result']
+        assert_equal(pkg['name'], 'annakarenina')
+        missing_keys = set(('title', 'groups')) - set(pkg.keys())
+        assert not missing_keys, missing_keys
+
+    def test_01_package_show_with_jsonp(self):
+        anna_id = model.Package.by_name(u'annakarenina').id
+        postparams = '%s=1' % json.dumps({'id': anna_id})
+        res = self.app.post('/api/action/package_show?callback=jsoncallback', params=postparams)
+
+        assert re.match('jsoncallback\(.*\);', res.body), res
+        # Unwrap JSONP callback (we want to look at the data).
+        msg = res.body[len('jsoncallback')+1:-2]
+        res_dict = json.loads(msg)
+        assert_equal(res_dict['success'], True)
+        assert_equal(res_dict['help'], None)
+        pkg = res_dict['result']
+        assert_equal(pkg['name'], 'annakarenina')
+        missing_keys = set(('title', 'groups')) - set(pkg.keys())
+        assert not missing_keys, missing_keys
+                
     def test_02_package_autocomplete(self):
-        postparams = '%s=1' % json.dumps({'q':'a'})
+        postparams = '%s=1' % json.dumps({'q':'war'})
         res = self.app.post('/api/action/package_autocomplete', params=postparams)
         res_obj = json.loads(res.body)
         assert res_obj['success'] == True
-        assert res_obj['result'][0]['name'] == 'annakarenina'
+        pprint(res_obj['result'][0]['name'])
+        assert res_obj['result'][0]['name'] == 'warandpeace'
 
     def test_03_create_update_package(self):
 
@@ -97,6 +125,25 @@ class TestAction(WsgiAppCase):
         package_created.pop('revision_timestamp')
         assert package_updated == package_created#, (pformat(json.loads(res.body)), pformat(package_created['result']))
 
+    def test_18_create_package_not_authorized(self):
+
+        package = {
+            'extras': [{'key': u'original media','value': u'"book"'}],
+            'license_id': u'other-open',
+            'maintainer': None,
+            'maintainer_email': None,
+            'name': u'annakareninanew_not_authorized',
+            'notes': u'Some test now',
+            'tags': [{'name': u'russian'}, {'name': u'tolstoy'}],
+            'title': u'A Novel By Tolstoy',
+            'url': u'http://www.annakarenina.com',
+        }
+
+        wee = json.dumps(package)
+        postparams = '%s=1' % json.dumps(package)
+        res = self.app.post('/api/action/package_create', params=postparams,
+                                     status=self.STATUS_403_ACCESS_DENIED)
+
     def test_04_user_list(self):
         postparams = '%s=1' % json.dumps({})
         res = self.app.post('/api/action/user_list', params=postparams)
@@ -109,6 +156,7 @@ class TestAction(WsgiAppCase):
         assert not 'apikey' in res_obj['result'][0]
 
     def test_05_user_show(self):
+        # Anonymous request
         postparams = '%s=1' % json.dumps({'id':'annafan'})
         res = self.app.post('/api/action/user_show', params=postparams)
         res_obj = json.loads(res.body)
@@ -117,12 +165,33 @@ class TestAction(WsgiAppCase):
         result = res_obj['result']
         assert result['name'] == 'annafan'
         assert result['about'] == 'I love reading Annakarenina. My site: <a href="http://anna.com">anna.com</a>'
-        assert 'apikey' in result
         assert 'activity' in result
         assert 'created' in result
         assert 'display_name' in result
         assert 'number_administered_packages' in result
         assert 'number_of_edits' in result
+        assert not 'apikey' in result
+        assert not 'reset_key' in result
+
+        # Same user can see his api key
+        res = self.app.post('/api/action/user_show', params=postparams,
+                            extra_environ={'Authorization': str(self.normal_user.apikey)})
+
+        res_obj = json.loads(res.body)
+        result = res_obj['result']
+        assert result['name'] == 'annafan'
+        assert 'apikey' in result
+        assert 'reset_key' in result
+
+        # Sysadmin user can see everyone's api key
+        res = self.app.post('/api/action/user_show', params=postparams,
+                            extra_environ={'Authorization': str(self.sysadmin_user.apikey)})
+
+        res_obj = json.loads(res.body)
+        result = res_obj['result']
+        assert result['name'] == 'annafan'
+        assert 'apikey' in result
+        assert 'reset_key' in result
 
     def test_05_user_show_edits(self):
         postparams = '%s=1' % json.dumps({'id':'tester'})
@@ -216,6 +285,7 @@ class TestAction(WsgiAppCase):
     def test_09_user_create(self):
         user_dict = {'name':'test_create_from_action_api',
                       'about': 'Just a test user',
+                      'email': 'me@test.org',
                       'password':'testpass'}
 
         postparams = '%s=1' % json.dumps(user_dict)
@@ -246,6 +316,7 @@ class TestAction(WsgiAppCase):
             'error': {
                 '__type': 'Validation Error',
                 'name': ['Missing value'],
+                'email': ['Missing value'],
                 'password': ['Missing value']
             },
             'help': 'Creates a new user',
@@ -254,6 +325,7 @@ class TestAction(WsgiAppCase):
 
     def test_11_user_create_wrong_password(self):
         user_dict = {'name':'test_create_from_action_api_2',
+                'email':'me@test.org',
                       'password':'tes'} #Too short
 
         postparams = '%s=1' % json.dumps(user_dict)
@@ -274,10 +346,12 @@ class TestAction(WsgiAppCase):
     def test_12_user_update(self):
         normal_user_dict = {'id': self.normal_user.id,
                             'fullname': 'Updated normal user full name',
+                            'email': 'me@test.org',
                             'about':'Updated normal user about'}
 
         sysadmin_user_dict = {'id': self.sysadmin_user.id,
                             'fullname': 'Updated sysadmin user full name',
+                            'email': 'me@test.org',
                             'about':'Updated sysadmin user about'} 
 
         #Normal users can update themselves
@@ -451,3 +525,41 @@ class TestAction(WsgiAppCase):
         res_obj = json.loads(res.body)
         assert_equal(res_obj, u'Bad request - Action name not known: bad_action_name')
 
+    def test_19_update_resource(self):
+        package = {
+            'name': u'annakareninanew',
+            'resources': [{
+                'alt_url': u'alt123',
+                'description': u'Full text.',
+                'extras': {u'alt_url': u'alt123', u'size': u'123'},
+                'format': u'plain text',
+                'hash': u'abc123',
+                'position': 0,
+                'url': u'http://www.annakarenina.com/download/'
+            }],
+            'title': u'A Novel By Tolstoy',
+            'url': u'http://www.annakarenina.com',
+        }
+
+        postparams = '%s=1' % json.dumps(package)
+        res = self.app.post('/api/action/package_create', params=postparams,
+                            extra_environ={'Authorization': 'tester'})
+        package_created = json.loads(res.body)['result']
+
+        resource_created = package_created['resources'][0]
+        new_resource_url = u'http://www.annakareinanew.com/download/' 
+        resource_created['url'] = new_resource_url
+        postparams = '%s=1' % json.dumps(resource_created)
+        res = self.app.post('/api/action/resource_update', params=postparams,
+                            extra_environ={'Authorization': 'tester'})
+        
+        resource_updated = json.loads(res.body)['result']
+        assert resource_updated['url'] == new_resource_url, resource_updated
+
+        resource_updated.pop('url')
+        resource_updated.pop('revision_id')
+        resource_updated.pop('revision_timestamp')
+        resource_created.pop('url')
+        resource_created.pop('revision_id')
+        resource_created.pop('revision_timestamp')
+        assert resource_updated == resource_created
