@@ -1,10 +1,13 @@
-from ckan.tests import *
-import ckan.model as model
-from base import FunctionalTestCase
+from nose.tools import assert_equal
 
 from ckan.plugins import SingletonPlugin, implements, IGroupController
 from ckan import plugins
-from ckan.tests import search_related
+import ckan.model as model
+from ckan.lib.create_test_data import CreateTestData
+
+from ckan.tests import *
+from base import FunctionalTestCase
+from ckan.tests import search_related, is_search_supported
 
 class MockGroupControllerPlugin(SingletonPlugin):
     implements(IGroupController)
@@ -42,21 +45,24 @@ class TestGroup(FunctionalTestCase):
     def teardown_class(self):
         model.repo.rebuild_db()
 
-    @search_related
     def test_mainmenu(self):
+        # the home page does a package search so have to skip this test if
+        # search is not supported
+        if not is_search_supported():
+            from nose import SkipTest
+            raise SkipTest("Search not supported")
+
         offset = url_for(controller='home', action='index')
         res = self.app.get(offset)
         assert 'Groups' in res, res
         assert 'Groups</a>' in res, res
         res = res.click(href='/group', index=0)
-        assert '<h2>Groups</h2>' in res, res
+        assert 'Groups of' in res, res
 
     def test_index(self):
         offset = url_for(controller='group')
         res = self.app.get(offset)
-        assert '<h2>Groups</h2>' in res, res
-        group_count = model.Session.query(model.Group).count()
-        assert 'There are %s groups.' % group_count in self.strip_tags(res)
+        assert '<h1 class="page_heading">Groups' in res, res
         groupname = 'david'
         group = model.Group.by_name(unicode(groupname))
         group_title = group.title
@@ -77,13 +83,18 @@ class TestGroup(FunctionalTestCase):
             main_res = self.main_div(res)
             assert title in res, res
             #assert 'edit' not in main_res, main_res
-            assert 'Administrators:' in main_res, main_res
+            assert 'Administrators' in res, res
             assert 'russianfan' in main_res, main_res
             assert name in res, res
-            assert 'There are 2 packages in this group' in self.strip_tags(main_res), main_res
+            assert 'There are 2 datasets in this group' in self.strip_tags(main_res), main_res
             pkg = model.Package.by_name(pkgname)
             res = res.click(pkg.title)
-            assert '%s - Data Packages' % pkg.title in res
+            assert '%s - Datasets' % pkg.title in res
+
+    def test_read_non_existent(self):
+        name = u'group_does_not_exist'
+        offset = url_for(controller='group', action='read', id=name)
+        res = self.app.get(offset, status=404)
 
     def test_read_plugin_hook(self):
         plugin = MockGroupControllerPlugin()
@@ -107,16 +118,16 @@ class TestGroup(FunctionalTestCase):
     def test_new_page(self):
         offset = url_for(controller='group', action='new')
         res = self.app.get(offset, extra_environ={'REMOTE_USER': 'russianfan'})
-        assert 'Create a new group' in res, res
+        assert 'Add A Group' in res, res
 
 
 class TestEdit(FunctionalTestCase):
-    groupname = u'david'
 
     @classmethod
     def setup_class(self):
         model.Session.remove()
         CreateTestData.create()
+        self.groupname = u'david'
         self.packagename = u'testpkg'
         model.repo.new_revision()
         model.Session.add(model.Package(name=self.packagename))
@@ -135,22 +146,14 @@ class TestEdit(FunctionalTestCase):
         res = res.follow()
         assert res.request.url.startswith('/user/login')
 
-    def test_1_read_allowed_for_admin(self):
-        offset = url_for(controller='group', action='edit', id=self.groupname)
-        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
-        assert 'Edit Group: %s' % self.groupname in res, res
-        
     def test_2_edit(self):
+        group = model.Group.by_name(self.groupname)
         offset = url_for(controller='group', action='edit', id=self.groupname)
         print offset
         res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
-        assert 'Edit Group: %s' % self.groupname in res, res
-
-        print res
-        
+        assert 'Edit: %s' % group.title in res, res
 
         form = res.forms['group-edit']
-        group = model.Group.by_name(self.groupname)
         titlefn = 'title'
         descfn = 'description'
         newtitle = 'xxxxxxx'
@@ -174,11 +177,11 @@ Ho ho ho
         assert group.title == newtitle, group
         assert group.description == newdesc, group
 
-        # now look at packages
+        # now look at datasets
         assert len(group.packages) == 3
 
     def test_3_edit_form_has_new_package(self):
-        # check for package in autocomplete
+        # check for dataset in autocomplete
         offset = url_for(controller='package', action='autocomplete', q='an')
         res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
         assert 'annakarenina' in res, res
@@ -198,6 +201,30 @@ Ho ho ho
         assert 'annakarenina' in res, res
         assert 'newone' in res
         
+    def test_4_new_duplicate_package(self):
+        prefix = ''
+
+        # Create group
+        group_name = u'testgrp4'
+        CreateTestData.create_groups([{'name': group_name,
+                                       'packages': [self.packagename]}],
+                                     admin_user_name='russianfan')
+
+        # Add same package again
+        offset = url_for(controller='group', action='edit', id=group_name)
+        res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
+        fv = res.forms['group-edit']
+        fv['packages__1__name'] = self.packagename
+        res = fv.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
+        res = res.follow()
+        assert group_name in res, res
+        model.Session.remove()
+
+        # check package only added to the group once
+        group = model.Group.by_name(group_name)
+        pkg_names = [pkg.name for pkg in group.packages]
+        assert_equal(pkg_names, [self.packagename])
+
     def test_edit_plugin_hook(self):
         plugin = MockGroupControllerPlugin()
         plugins.load(plugin)
@@ -210,6 +237,11 @@ Ho ho ho
         assert plugin.calls['edit'] == 1, plugin.calls
         plugins.unload(plugin)
 
+    def test_edit_non_existent(self):
+        name = u'group_does_not_exist'
+        offset = url_for(controller='group', action='edit', id=name)
+        res = self.app.get(offset, status=404)
+
 class TestNew(FunctionalTestCase):
     groupname = u'david'
 
@@ -242,10 +274,10 @@ class TestNew(FunctionalTestCase):
         group_title = u'Test Title'
         group_description = u'A Description'
 
-        # Open 'new group' page
+        # Open 'Add A Group' page
         offset = url_for(controller='group', action='new')
         res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
-        assert 'New Group' in res, res
+        assert 'Add A Group' in res, res
         fv = res.forms['group-edit']
         assert fv[prefix+'name'].value == '', fv.fields
         assert fv[prefix+'title'].value == ''
@@ -270,28 +302,27 @@ class TestNew(FunctionalTestCase):
         pkg = model.Package.by_name(self.packagename)
         assert group.packages == [pkg]
 
-    def test_3_new_duplicate(self):
+    def test_3_new_duplicate_group(self):
         prefix = ''
 
         # Create group
         group_name = u'testgrp1'
         offset = url_for(controller='group', action='new')
         res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
-        assert 'New Group' in res, res
+        assert 'Add A Group' in res, res
         fv = res.forms['group-edit']
         assert fv[prefix+'name'].value == '', fv.fields
         fv[prefix+'name'] = group_name
         res = fv.submit('save', status=302, extra_environ={'REMOTE_USER': 'russianfan'})
         res = res.follow()
         assert group_name in res, res
-        assert 'No Title' in res, res
         model.Session.remove()
 
         # Create duplicate group
         group_name = u'testgrp1'
         offset = url_for(controller='group', action='new')
         res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER': 'russianfan'})
-        assert 'New Group' in res, res
+        assert 'Add A Group' in res, res
         fv = res.forms['group-edit']
         assert fv[prefix+'name'].value == '', fv.fields
         fv[prefix+'name'] = group_name
