@@ -1,4 +1,5 @@
-from pylons import c
+from pylons import c, session
+from pylons.i18n import set_lang
 
 from ckan.lib.create_test_data import CreateTestData
 from ckan.controllers.home import HomeController
@@ -7,11 +8,12 @@ import ckan.model as model
 from ckan.tests import *
 from ckan.tests.html_check import HtmlCheckMethods
 from ckan.tests.pylons_controller import PylonsTestCase
-from ckan.tests import search_related
+from ckan.tests import search_related, setup_test_search_index
 
 class TestHomeController(TestController, PylonsTestCase, HtmlCheckMethods):
     @classmethod
     def setup_class(cls):
+        setup_test_search_index()
         PylonsTestCase.setup_class()
         model.repo.init_db()
         CreateTestData.create()
@@ -20,27 +22,44 @@ class TestHomeController(TestController, PylonsTestCase, HtmlCheckMethods):
     def teardown_class(self):
         model.repo.rebuild_db()
 
+    def clear_language_setting(self):
+        self.app.cookies = {}
+
     def test_home_page(self):
         offset = url_for('home')
         res = self.app.get(offset)
-        assert 'Packages' in res
+        assert 'Add a dataset' in res
+        assert 'Could not change language' not in res
 
     def test_calculate_etag_hash(self):
+        # anything that changes the home page appearance should change the
+        # etag hash
         c.user = 'test user'
         get_hash = HomeController._home_cache_key
-        hash_1 = get_hash()
-        hash_2 = get_hash()
-        self.assert_equal(hash_1, hash_2)
+        hashes = [get_hash(), get_hash()]
+        self.assert_equal(hashes[0], hashes[1])
 
+        def assert_hash_changed(hashes):
+            current_hash = get_hash()
+            assert current_hash != hashes[-1]
+            hashes.append(current_hash)
+
+        # login as a different user
         c.user = 'another user'
-        hash_3 = get_hash()
-        assert hash_2 != hash_3
+        assert_hash_changed(hashes)
 
-        model.repo.new_revision()
-        model.Session.add(model.Package(name=u'test_etag'))
+        # add a package to a group
+        rev = model.repo.new_revision()
+        model.Group.by_name(u'roger').add_package_by_name(u'warandpeace')
         model.repo.commit_and_remove()
-        hash_4 = get_hash()
-        assert hash_3 != hash_4
+        assert_hash_changed(hashes)
+
+        # flash message is not cached, but this is done in ckan/lib/cache
+        
+        # I can't get set_lang to work and deliver correct
+        # result to get_lang, so leaving it commented
+##        set_lang('fr')
+##        assert_hash_changed(hashes)
 
     @search_related
     def test_packages_link(self):
@@ -48,49 +67,55 @@ class TestHomeController(TestController, PylonsTestCase, HtmlCheckMethods):
         res = self.app.get(offset)
         res.click('Search', index=0)
         
-    def test_tags_link(self):
-        offset = url_for('home')
-        res = self.app.get(offset)
-        res.click('Tags', index=0)
-        
     def test_404(self):
         offset = '/some_nonexistent_url'
         res = self.app.get(offset, status=404)
 
-    def test_license(self):
-        offset = url_for('license')
-        res = self.app.get(offset)
-        assert 'License' in res
-
     def test_guide(self):
         url = url_for('guide')
         assert url == 'http://wiki.okfn.org/ckan/doc/'
 
-    @search_related
-    def test_search_packages(self):
+    def test_template_head_end(self):
         offset = url_for('home')
         res = self.app.get(offset)
-        form = res.forms['package-search']
-        form['q'] =  'anna'
-        results_page = form.submit()
-        assert 'Search - ' in results_page, results_page
-        assert '>0<' in results_page, results_page
-    
+        assert 'ckan.template_head_end = <link rel="stylesheet" href="TEST_TEMPLATE_HEAD_END.css" type="text/css"> '
+
     def test_template_footer_end(self):
         offset = url_for('home')
         res = self.app.get(offset)
         assert '<strong>TEST TEMPLATE_FOOTER_END TEST</strong>'
 
-    # DISABLED because this is not on home page anymore
-    def _test_register_new_package(self):
+## Browser lang detection disabled - see #1452
+##    def test_locale_detect(self):
+##        offset = url_for('home')
+##        self.clear_language_setting()
+##        res = self.app.get(offset, headers={'Accept-Language': 'de,pt-br,en'})
+##        try:
+##            assert 'Willkommen' in res.body, res.body
+##        finally:
+##            self.clear_language_setting()
+
+    def test_locale_negotiate(self):
         offset = url_for('home')
-        res = self.app.get(offset)
-        form = res.forms[1]
-        form['title'] =  'test title'
-        results_page = form.submit()
-        assert 'Register a New Package' in results_page, results_page
-        assert '<input id="Package--title" name="Package--title" size="40" type="text" value="test title">' in results_page, results_page
-        
+        self.clear_language_setting()
+        res = self.app.get(offset, headers={'Accept-Language': 'fr-ca'})
+        # Request for French with Canadian territory should negotiate to
+        # just 'fr'
+        try:
+            assert 'propos' in res.body, res.body
+        finally:
+            self.clear_language_setting()
+
+    def test_locale_negotiate_pt(self):
+        offset = url_for('home')
+        self.clear_language_setting()
+        res = self.app.get(offset, headers={'Accept-Language': 'pt'})
+        # Request for Portuguese should find pt_BR because of our alias hack
+        try:
+            assert 'Bem-vindo' in res.body, res.body
+        finally:
+            self.clear_language_setting()
+
     def test_locale_change(self):
         offset = url_for('home')
         res = self.app.get(offset)
@@ -99,7 +124,7 @@ class TestHomeController(TestController, PylonsTestCase, HtmlCheckMethods):
             res = res.follow()
             assert 'Willkommen' in res.body
         finally:
-            res = res.click('English')
+            self.clear_language_setting()
 
     def test_locale_change_invalid(self):
         offset = url_for(controller='home', action='locale', locale='')
@@ -134,6 +159,19 @@ class TestHomeController(TestController, PylonsTestCase, HtmlCheckMethods):
             res = res.goto(href)
             assert res.status == 200, res.status # doesn't redirect
         finally:
-            offset = url_for('home')
-            res = self.app.get(offset)
-            res = res.click('English')
+            self.clear_language_setting()
+
+class TestDatabaseNotInitialised(TestController):
+    @classmethod
+    def setup_class(cls):
+        PylonsTestCase.setup_class()
+        model.repo.clean_db()
+
+    @classmethod
+    def teardown_class(self):
+        model.repo.rebuild_db()
+
+    def test_home_page(self):
+        offset = url_for('home')
+        res = self.app.get(offset, status=503)
+        assert 'This site is currently off-line. Database is not initialised.' in res
