@@ -53,9 +53,9 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
                                  'http://anna.com',
                                  'target="_blank"',
                                  'rel="nofollow"')
-        assert 'Edit' not in main_res, main_res
+        assert 'Edit Profile' not in main_res, main_res
         assert 'Number of edits:</strong> 3' in res, res
-        assert 'Number of packages administered:</strong> 1' in res, res
+        assert 'Number of datasets administered:</strong> 1' in res, res
         assert 'Revision History' in res, res
 
     def test_user_read_without_id(self):
@@ -81,7 +81,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         main_res = self.main_div(res)
         assert 'annafan' in res, res
         assert 'My Account' in main_res, main_res
-        assert 'Edit' in main_res, main_res
+        assert 'Edit Profile' in main_res, main_res
 
     def test_user_read_about_unfinished(self):
         user = model.User.by_name(u'unfinisher')
@@ -158,8 +158,8 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         offset = url_for(controller='user', action='login')
         res = self.app.get(offset)
         fv = res.forms['login']
-        fv['login'] = username
-        fv['password'] = password
+        fv['login'] = str(username)
+        fv['password'] = str(password)
         res = fv.submit()
 
         # check cookies set
@@ -176,7 +176,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         assert_equal(res.header('Location'), 'http://localhost/user/testlogin')
         res = res.follow()
         assert_equal(res.status, 200)
-        assert 'Welcome back, testlogin' in res.body
+        assert 'testlogin is now logged in' in res.body
         assert 'My Account' in res.body
         
         # check user object created
@@ -262,6 +262,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         username = 'testcreate'
         fullname = u'Test Create'
         password = u'testpassword'
+        email = u'test@test.org'
         assert not model.User.by_name(unicode(username))
         rev_id_before_test = model.repo.youngest_revision().id
 
@@ -272,6 +273,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         fv = res.forms['user-edit']
         fv['name'] = username
         fv['fullname'] = fullname
+        fv['email'] = email
         fv['password1'] = password
         fv['password2'] = password
         res = fv.submit('save')
@@ -294,6 +296,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         assert user
         assert_equal(user.name, username)
         assert_equal(user.fullname, fullname)
+        assert_equal(user.email, email)
         assert user.password
         
         # no revision should be created - User is not revisioned
@@ -314,6 +317,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         username = u'testcreate4'
         fullname = u'Test Create\xc2\xa0'
         password = u'testpassword\xc2\xa0'
+        email = u'me@test.org'
         assert not model.User.by_name(username)
 
         offset = url_for(controller='user', action='register')
@@ -323,6 +327,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         fv = res.forms['user-edit']
         fv['name'] = username
         fv['fullname'] = fullname.encode('utf8')
+        fv['email'] = email
         fv['password1'] = password.encode('utf8')
         fv['password2'] = password.encode('utf8')
         res = fv.submit('save')
@@ -465,7 +470,8 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         user = model.User.by_name(unicode(username))
         if not user:
             model.Session.add(model.User(name=unicode(username), about=about,
-                                         password='letmein'))
+                email=u'me@test.org',
+                password='letmein'))
             model.repo.commit_and_remove()
             user = model.User.by_name(unicode(username))
         rev_id_before_test = model.repo.youngest_revision().id
@@ -473,6 +479,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         # edit
         new_about = u'Changed about'
         new_password = u'testpass'
+        new_openid = u'http://mynewopenid.com/'
         offset = url_for(controller='user', action='edit', id=user.id)
         res = self.app.get(offset, status=200, extra_environ={'REMOTE_USER':username})
         main_res = self.main_div(res)
@@ -480,15 +487,9 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         assert about in main_res, main_res
         fv = res.forms['user-edit']
         fv['about'] = new_about
+        fv['openid'] = new_openid
         fv['password1'] = new_password
         fv['password2'] = new_password
-        res = fv.submit('preview', extra_environ={'REMOTE_USER':username})
-        
-        # preview
-        main_res = self.main_div(res)
-        assert 'Edit User: testedit' in main_res, main_res
-        in_preview = main_res[main_res.find('Preview'):]
-        assert new_about in in_preview, in_preview
 
         # commit
         res = fv.submit('save', extra_environ={'REMOTE_USER':username})      
@@ -498,6 +499,9 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         assert 'testedit' in main_res, main_res
         assert new_about in main_res, main_res
 
+        updated_user = model.User.by_name(unicode(username))
+        assert_equal(updated_user.openid, new_openid)
+
         # read, not logged in
         offset = url_for(controller='user', action='read', id=user.id)
         res = self.app.get(offset, status=200)
@@ -515,7 +519,8 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         user = model.User.by_name(unicode(username))
         if not user:
             model.Session.add(model.User(name=unicode(username), about=about,
-                                         password='letmein'))
+                email=u'me@test.org',
+                password='letmein'))
             model.repo.commit_and_remove()
             user = model.User.by_name(unicode(username))
 
@@ -533,14 +538,6 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         fv['password1'] = ''
         fv['password2'] = ''
 
-        res = fv.submit('preview', extra_environ={'REMOTE_USER':username})
-        
-        # preview
-        main_res = self.main_div(res)
-        assert 'Edit User: testedit2' in main_res, main_res
-        in_preview = main_res[main_res.find('Preview'):]
-        assert new_about in in_preview, in_preview
-
         # commit
         res = fv.submit('save', extra_environ={'REMOTE_USER':username})      
         assert res.status == 302, self.main_div(res).encode('utf8')
@@ -603,7 +600,6 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         assert 'Edit User: ' in main_res, main_res
         assert 'Test About &lt;a href="http://spamsite.net"&gt;spamsite&lt;/a&gt;' in main_res, main_res
         fv = res.forms['user-edit']
-        res = fv.submit('preview', extra_environ={'REMOTE_USER':username})
         # commit
         res = fv.submit('save', extra_environ={'REMOTE_USER':username})      
         assert res.status == 200, res.status
@@ -615,8 +611,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         # comes back as a params like this:
         # e.g. /user/login?error=Error%20in%20discovery:%20Error%20fetching%20XRDS%20document:%20(6,%20%22Couldn't%20resolve%20host%20'mysite.myopenid.com'%22)
         res = self.app.get("/user/login?error=Error%20in%20discovery:%20Error%20fetching%20XRDS%20document:%20(6,%20%22Couldn't%20resolve%20host%20'mysite.myopenid.com'%22")
-        main_res = self.main_div(res)
-        assert "Couldn't resolve host" in main_res, main_res
+        assert "Couldn&#39;t resolve host" in res, res
 
     def _login_openid(self, res):
         # this requires a valid account on some openid provider
@@ -645,8 +640,7 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         fv = res.forms['user-password-reset']
         fv['user'] = 'unknown'
         res = fv.submit()
-        main_res = self.main_div(res)
-        assert 'No such user: unknown' in main_res, main_res # error
+        assert 'No such user: unknown' in res, res # error
 
     def test_request_reset_user_password_using_search(self):
         CreateTestData.create_user(name='larry1', email='kittens@john.com')
@@ -664,24 +658,21 @@ class TestUserController(FunctionalTestCase, HtmlCheckMethods, PylonsTestCase, S
         fv = res.forms['user-password-reset']
         fv['user'] = 'kittens'
         res = fv.submit()
-        main_res = self.main_div(res)
-        assert '"kittens" matched several users' in main_res, main_res
-        assert 'larry1' not in main_res, main_res
-        assert 'larry2' not in main_res, main_res
+        assert '&#34;kittens&#34; matched several users' in res, res
+        assert 'larry1' not in res, res
+        assert 'larry2' not in res, res
 
         res = self.app.get(offset)
         fv = res.forms['user-password-reset']
         fv['user'] = ''
         res = fv.submit()
-        main_res = self.main_div(res)
-        assert 'No such user:' in main_res, main_res
+        assert 'No such user:' in res, res
 
         res = self.app.get(offset)
         fv = res.forms['user-password-reset']
         fv['user'] = 'l'
         res = fv.submit()
-        main_res = self.main_div(res)
-        assert 'No such user:' in main_res, main_res
+        assert 'No such user:' in res, res
 
     def test_reset_user_password_link(self):
         # Set password
